﻿angular.module('mobile-app.controllers').controller('AppLandingCtrl', function ($scope, $stateParams, $timeout, metadataServices, telemetry, $label, $page) {
    let appId = $stateParams.appId;
    $scope.appState = "loading";
    let lastHash;

    function applyAppMetadata(state) {
        // Applies the app metadata to the scope
        return function applyAppMetadataResult(appMetadata) {
            if (!appMetadata || (lastHash !== undefined && appMetadata.$hash === lastHash)) {
                return;
            }

            // Update the hash
            lastHash = appMetadata.$hash;

            $timeout(function () {
                $scope.appState = state;

                let args = {
                    appId: appId,
                    pageId: appMetadata.WorkspacePageId
                };
                let page = $page.createForm(args, $scope);
                page.onRefresh.subscribe(() => {
                    // Refresh app metadata only if it is an explicit forced refresh
                    getApp(0);
                    var workSpacePropertyName = telemetry.events.workspaceRefreshed.TelemetryProperties.workspaceName.Name;
                    telemetry.trackEvent(telemetry.events.workspaceRefreshed.Name, { [workSpacePropertyName]: appMetadata.Title });
                });
                let pageGetStateFunc = page.getLoadState;
                page.getLoadState = function () {
                    if ($scope.appState !== "loaded") {
                        return state;
                    }
                    else {
                        return pageGetStateFunc.bind(page)();
                    }
                }
                let pageGetErrorFunc = page.getLoadErrorCode;
                page.getErrorCode = function () {
                    return $scope.appErrorCode || pageGetErrorFunc.bind(page)();
                }
                var workSpacePropertyName = telemetry.events.workspaceLoaded.TelemetryProperties.workspaceName.Name;
                telemetry.trackEvent(telemetry.events.workspaceLoaded.Name, { [workSpacePropertyName]: appMetadata.Title });
            });
        };
    }

    function errorHandler(error) {
        $scope.appState = "error";
        $scope.appErrorCode = error;
        $scope.$apply();
        console.log("Error retrieving App: " + appId);
        var workSpaceAppIdPropertyName = telemetry.events.workspaceError.TelemetryProperties.workspaceAppId.Name;
        let workSpaceErrorPropertyName = telemetry.events.workspaceError.TelemetryProperties.error.Name;
        telemetry.trackEvent(telemetry.events.workspaceError.Name, { [workSpaceErrorPropertyName]: error, [workSpaceAppIdPropertyName]: appId });
    }

    function getApp(stalenessTime) {
        metadataServices.getApp(appId, (stalenessTime * 24 * 60 * 60)).then(
            applyAppMetadata("loaded"),
            errorHandler,
            applyAppMetadata("refreshing"));
    }

    getApp(7);/* 7 days of staleness is fine */

});
