﻿(function () {
    'use strict';

    angular.module('mobile-app.controllers').controller('AppListCtrl', function ($scope, $stateParams, $timeout, telemetry, metadataServices, cacheServices, $label) {
        $scope.appList = [];
        $scope.dataState = "loading";
        $scope.$label = $label;
        // Applies the app metadata to the scope
        function applyAppList(data) {
            $timeout(function () {
                $scope.appList = data;
                $scope.dataState = "loaded";
                var workspaceCountPropertyName = telemetry.events.dashboardLoaded.TelemetryProperties.workspaceCount.Name;
                telemetry.trackEvent(telemetry.events.dashboardLoaded.Name, { [workspaceCountPropertyName]: data.length });
                if (data == null || (data !== null && data.length == 0)) {
                    $scope.showWiki = true;
                }
                else {
                    $scope.showWiki = false;
                }
            });
        }

        function errorHandler(error) {
            $scope.dataState = "error";
            $scope.errorCode = error;
            $scope.$apply();
            console.log(error);
            let dashBoardErrorPropertyName = telemetry.events.dashboardError.TelemetryProperties.error.Name;
            telemetry.trackEvent(telemetry.events.dashboardError.Name, { [dashBoardErrorPropertyName]: error });
        }

        function loadAppList() {
            metadataServices.getAppList($stateParams.allowedStaleness || (7 * 24 * 60 * 60) /* 7 days of staleness is fine */)
                .then(applyAppList, function (error) {
                    errorHandler(error);
                }, applyAppList);
        };

        loadAppList();

        $scope.getIconClasses = function (appItem) {
            var classes = [];
            classes.push(appItem.colorName);

            var symbolName = appItem.iconName != "None" ? appItem.iconName : "List";
            classes.push(symbolName + "-symbol");
            return classes;
        };

        $scope.doRefresh = function () {
            if ($scope.dataState == "loading" || $scope.dataState == "refreshing") {
                // Don't issue a a request if we are allready getting data from the server.
                $scope.$broadcast('scroll.refreshComplete');
            }
            else {
                $scope.dataState = "refreshing";
                return metadataServices.getAppList(0)
                    .then(function (data) {
                        applyAppList(data);
                        $scope.$broadcast('scroll.refreshComplete');
                        telemetry.trackEvent(telemetry.events.dashboardRefreshed.Name);
                    }, function (error) {
                        errorHandler(error);
                    });
            }
        };

        $scope.$on("$ionicView.enter", function (event, data) {
            loadAppList();
            cacheServices.setData("$workspaceNameForTelemetry", "");
        });

        document.addEventListener("ax-companyChanged", function () {
            //When company is changed we need to refresh workspace list
            $scope.doRefresh();
        });
    });
})();
