(function () {
    'use strict';

    angular.module('mobile-app').component('calendar', {
        bindings: {
            data: '=',
            metadata: '='
        },

        controller: function ($scope, $stateParams, $state, controlBinding, $label, $accessibility) {
            $scope.$label = $label;

            var daysOfWeekName = [$label("Day_Of_Week_Sunday"), $label("Day_Of_Week_Monday"), $label("Day_Of_Week_Tuesday"),
            $label("Day_Of_Week_Wednesday"), $label("Day_Of_Week_Thursday"), $label("Day_Of_Week_Friday"), $label("Day_Of_Week_Saturday")];

            var monthsOfYear = [$label("Month_January"), $label("Month_February"), $label("Month_March"), $label("Month_April"),
            $label("Month_May"), $label("Month_June"), $label("Month_July"), $label("Month_August"), $label("Month_September"),
            $label("Month_October"), $label("Month_November"), $label("Month_December")];


            $scope.getListNavigationHandler = function (item) {
                var navigation;
                var listControl = $scope.$ctrl.metadata;
                if (listControl.DetailsPageId && item.CanonicalId) {
                    var navigation = {
                        to: 'app.detailPage',
                        params: {
                            appId: $stateParams.appId,
                            pageId: listControl.DetailsPageId,
                            pageContext: item.CanonicalId
                        },
                        options: undefined
                    };

                }

                if (navigation != null && listControl.OnNavigate) {
                    listControl.OnNavigate(navigation || {});
                }

                if (navigation != null && navigation.to) {
                    $state.go(navigation.to, navigation.params, navigation.options);
                }

                return true;
            };

            $scope.days = [];
            $scope.today = new Date();

            var getControl = function (listControl, controlName) {
                var found = false;
                for (var controlId in listControl.Children) {
                    var control = listControl.Children[controlId];
                    if (control.Name == controlName) {
                        found = true;
                        break;
                    }
                }
                return found ? control : null;
            }

            var getFullDateName = function (nameForDate) {
                return daysOfWeekName[nameForDate.getDay()] + ", " + monthsOfYear[nameForDate.getMonth()] + " " + nameForDate.getDate() +
                    ", " + (nameForDate.getYear() + 1900);
            }

            var getCardCalendarData = function () {
                var listControl = $scope.$ctrl.metadata;
                $scope.days = [];
                var listData = $scope.$ctrl.data;

                if (listControl.defaultDate) {
                    $scope.today = listControl.defaultDate;
                    listControl.defaultDate = undefined;
                }

                var today = $scope.today;
                var currentDay = today.getDay(); // (0-6) days of the week
                var sunday = new Date(today.getFullYear(), today.getMonth(), today.getDate());
                sunday.setDate(sunday.getDate() - currentDay);
                var sundayDayOfTheMonth = sunday.getDate(); //(1-31) days of the month
                var sundayDateStartOfWeek = sunday; // date for start of the week
                var saturdayDateEndOfWeek = new Date(sunday);
                saturdayDateEndOfWeek.setDate(saturdayDateEndOfWeek.getDate() + 6); // date for end of the week

                var calendarFields = {
                    groupByDateField: null,
                    titleFields: [],
                    subTitleFields: [],
                    altTitleFields: [],
                    durationInMinutesField: null,
                    durationInHoursField: null,
                    amountField: null,
                    activityCompletedHandler: null,
                    isTimerRunningHandler: null,
                    currentRunningTimeHandler: null
                };

                if (listControl.SetCalendarFields) {
                    listControl.SetCalendarFields(calendarFields);
                }

                if (listData && listData.length >= 0) {
                    var dateString = [];
                    var temp = new Date(sunday.getFullYear(), sunday.getMonth(), sunday.getDate());
                    // Each iteration of the loop is incrementing the value of temp by 1 so we
                    // always just add 1 again in each loop to get the day of the month number
                    // for the day.
                    for (var i = 0; i < 7; i++) {
                        temp.setDate(temp.getDate() + (i ? 1 : 0)); // first value in array is for sunday so add 0
                        dateString[i] = getFullDateName(temp);
                    }

                    // initialize the days array
                    $scope.days =
                        [
                            {
                                dayname: "Month",
                                items: [],
                                total: '',
                                displayTotal: ''
                            }
                        ];
                    // add sunday through saturday to the array
                    for (var i = 0; i < 7; i++) {
                        $scope.days[i + 1] = {
                            dayname: dateString[i],
                            items: [],
                            total: '',
                            displayTotal: ''
                        };
                    }

                    $scope.days[today.getDay() + 1].isToday = true;

                    if (calendarFields.groupByDateField != null && (calendarFields.titleFields.length > 0 || calendarFields.subTitleFields.length > 0 || calendarFields.altTitleFields.length > 0)) {
                        for (var i = 0; i < listData.length; i++) {
                            var item = listData[i];
                            var dateControl = getControl(listControl, calendarFields.groupByDateField);
                            var dateString = dateControl != null ? controlBinding.getControlValue(item, dateControl) : '';
                            if (dateString instanceof Date) {
                                dateString = dateString.toISOString();
                            }

                            if (dateString) {
                                var dateVal = undefined;
                                // Format like 2001-01-01
                                if ($scope.$ctrl.metadata && $scope.$ctrl.metadata.matchExactDate) {
                                    var parts = dateString.substring(0, 10).split('-');
                                    dateVal = new Date(parts[0], parts[1] - 1, parts[2]);
                                }
                                else {
                                    dateVal = new Date(Date.parse(dateString)); // See if date is in current week
                                }

                                if (dateVal.getYear() == today.getYear() && dateVal.getMonth() == today.getMonth() && dateVal >= sundayDateStartOfWeek && dateVal <= saturdayDateEndOfWeek) {
                                    var groupedDay = dateVal.getDay() + 1; // the day of the week that the current item/activity will be grouped into

                                    // create a simplified data model to bind to (instead of using item directly)
                                    var activity = {
                                        Name: "",
                                        SubName: "",
                                        AltName: "",
                                        Value: null,
                                        Completed: null,
                                        item: item,
                                        id: 0,
                                        displayValue: '',
                                        isTimerRunning: false
                                    }

                                    activity.id = item.CanonicalId;
                                    for (var fieldIndex in calendarFields.titleFields) {
                                        var titleControl = getControl(listControl, calendarFields.titleFields[fieldIndex]);
                                        var value = controlBinding.getControlValue(item, titleControl)
                                        activity.Name += value != null ? value + " " : "";
                                    }
                                    activity.Name = activity.Name.trim();


                                    for (var fieldIndex in calendarFields.subTitleFields) {
                                        var subTitleControl = getControl(listControl, calendarFields.subTitleFields[fieldIndex]);
                                        var value = controlBinding.getControlValue(item, subTitleControl);
                                        activity.SubName += value != null ? value + " " : "";
                                    }
                                    activity.SubName = activity.SubName.trim();

                                    // Only set the values for the AltName if the Name and SubName are both empty
                                    if (calendarFields.altTitleFields.length > 0 && activity.Name.length == 0 && activity.SubName.length == 0) {
                                        for (var fieldIndex in calendarFields.altTitleFields) {
                                            var altTitleControl = getControl(listControl, calendarFields.altTitleFields[fieldIndex]);
                                            var value = controlBinding.getControlValue(item, altTitleControl);
                                            activity.AltName += value != null ? value + " " : "";
                                        }
                                    }
                                    activity.AltName = activity.AltName.trim();

                                    // Handle Time which is expressed in Minutes or Hours
                                    var useTimeFormatting = false;
                                    if (calendarFields.durationInMinutesField != null) {
                                        var durationControl = getControl(listControl, calendarFields.durationInMinutesField);
                                        var value = controlBinding.getControlValue(item, durationControl);
                                        activity.Value = (value !== undefined && Number(value)) ? value : "";
                                        useTimeFormatting = true;
                                    }
                                    if (calendarFields.durationInHoursField != null) {
                                        var durationControl = getControl(listControl, calendarFields.durationInHoursField);
                                        var value = controlBinding.getControlValue(item, durationControl);
                                        activity.Value = (value !== undefined && Number(value)) ? value * 60 : "";
                                        useTimeFormatting = true;
                                    }
                                    if (calendarFields.amountField != null) {
                                        var amountControl = getControl(listControl, calendarFields.amountField);
                                        var value = controlBinding.getControlValue(item, amountControl);
                                        activity.Value = (value !== undefined && Number(value)) ? value : "";
                                    }
                                    activity.displayValue = formatValue(activity.Value, useTimeFormatting, listControl.FormatValueHandler);

                                    // activity is marked as completed if the activityCompletedHandler function returns true
                                    // Not supported in this release
                                    // activity.Completed = calendarFields.activityCompletedHandler ? calendarFields.activityCompletedHandler(item, calendarFieldToIdMap) : false;

                                    // activity duration display is modified to draw user attention to that the timer for the activity is running.
                                    activity.isTimerRunning = calendarFields.isTimerRunningHandler ? calendarFields.isTimerRunningHandler(item) : false;
                                    // if the timer is running ask for the current time on the timer to display on the calendar view
                                    if (activity.isTimerRunning) {
                                        var currentlyRunningTimeInMs = calendarFields.currentRunningTimeHandler ? calendarFields.currentRunningTimeHandler(item) : 0;
                                        activity.Value = Math.floor(Math.floor(Number(currentlyRunningTimeInMs) / 1000) / 60);
                                        activity.displayValue = formatValue(activity.Value, true, listControl.FormatValueHandler);
                                    }

                                    // calculate the total for the day
                                    if ($scope.days[groupedDay].total == '') {
                                        $scope.days[groupedDay].total = 0; // this day has at least one activity. so change from empty string to number
                                    }
                                    $scope.days[groupedDay].total += Number(activity.Value);
                                    $scope.days[groupedDay].items.push(activity);
                                }
                            }
                        }
                        for (var i = 0; i < $scope.days.length; i++) {
                            $scope.days[i].displayTotal = formatValue($scope.days[i].total, useTimeFormatting, listControl.FormatValueHandler);
                        }
                    }
                };
                return $scope.days;
            };

            var formatValue = function (valueToFormat, formatTime, FormatValueHandler) {
                if (FormatValueHandler) {
                    return FormatValueHandler(valueToFormat);
                } else if (formatTime) {
                    return formatMinutesDisplay(valueToFormat);
                } else {
                    return valueToFormat;
                }
            }

            var formatMinutesDisplay = function (minutesToFormat) {
                var hours = minutesToFormat / 60;
                hours = Math.floor(hours);  // round hours down as we will display minutes
                var minutes = minutesToFormat % 60;

                // if hours or minutes is 0 through 9 then add a zero infront of the number
                if (minutes < 10) {
                    minutes = "0" + minutes;
                }
                if (hours < 10) {
                    hours = "0" + hours;
                }

                var display = hours + ":" + minutes;
                return display;
            }

            $scope.month = { Headers: [], Weeks: [] };
            var getCardCalendarMonth = function () {
                $scope.month = { Headers: [], Weeks: [] };
                $scope.month.Headers = [
                    {abbreviature: $label("Day_Of_Week_Sunday_Abbreviation"), dayName: daysOfWeekName[0]},
                    {abbreviature: $label("Day_Of_Week_Monday_Abbreviation"), dayName: daysOfWeekName[1]},
                    {abbreviature: $label("Day_Of_Week_Tuesday_Abbreviation"), dayName: daysOfWeekName[2]},
                    {abbreviature: $label("Day_Of_Week_Wednesday_Abbreviation"), dayName: daysOfWeekName[3]},
                    {abbreviature: $label("Day_Of_Week_Thursday_Abbreviation"), dayName: daysOfWeekName[4]},
                    {abbreviature: $label("Day_Of_Week_Friday_Abbreviation"), dayName: daysOfWeekName[5]},
                    {abbreviature: $label("Day_Of_Week_Saturday_Abbreviation"), dayName: daysOfWeekName[6]}];

                var today = $scope.today;

                // Figure out what the first day to show on the calendar is
                // note that the first day of the current month is not the same as the first day shown on the calendar
                var FirstCalendarDay = new Date(today.getFullYear(), today.getMonth(), 1); // Temporarily set this to first day of month

                if (FirstCalendarDay.getDay() == 0) // First day of month is sunday
                {
                    //  First day of first week in current month is sunday
                    //  but we want to show the previous week as well so that we always have 6 weeks in the calendar
                    //  so set FirstCalendarDay as previous sunday

                    FirstCalendarDay.setDate(FirstCalendarDay.getDate() - 7); // First day of calendar is now correct

                }
                else {
                    //FirstCalendarDay begins the on the sunday before the first day of this month                    */
                    FirstCalendarDay.setDate(FirstCalendarDay.getDate() - FirstCalendarDay.getDay());	// First day of calendar is now correct
                }

                // Keep track of the number of days since the FirstCalendarDay in the calendar
                var daysAfter = 0;

                var currentDay = today.getDay(); // (0-6) days of the week
                var sundayForCurrentDate = new Date(today.getFullYear(), today.getMonth(), today.getDate());
                sundayForCurrentDate.setDate(sundayForCurrentDate.getDate() - currentDay);

                // We always show exactly 6 weeks in the calendar view
                for (var i = 0; i < 6; i++) {
                    var week = { Days: [], isCurrent: false };

                    // Figure out if this week is the current week based on today's date
                    // Week i begins on the date (1-31) that is 7x(i) days after the FirstCalendarDay
                    var startOfThisWeek = new Date(FirstCalendarDay);
                    startOfThisWeek.setDate(FirstCalendarDay.getDate() + 7 * i); // now start of week i is correct

                    week.isCurrent = (startOfThisWeek.getTime() == sundayForCurrentDate.getTime());

                    // Add the days to the week
                    for (var j = 0; j < 7; j++) {
                        var day = { DisplayDate: null, isToday: false, isInMonth: false, ActualDate: null, AccessibilityName: null };

                        var d = new Date(FirstCalendarDay);
                        d.setDate(FirstCalendarDay.getDate() + daysAfter);
                        daysAfter++;

                        day.DisplayDate = d.getDate();
                        day.isToday = (d.getDate() == today.getDate() && d.getMonth() == today.getMonth() && d.getFullYear() == today.getFullYear());
                        day.isInMonth = (d.getMonth() == today.getMonth());
                        day.ActualDate = d;
                        day.AccessibilityName = getFullDateName(d);
                        week.Days.push(day);
                    }
                    // restricts month array to only contain the current week. This is for use with the day view calendar.
                    if (week.isCurrent) {
                        $scope.month.Weeks.push(week);
                    }
                }
                return $scope.month;
            };

            $scope.moveToNextWeek = function () {
                var today = $scope.today;
                var dayNextWeek = new Date(today.getFullYear(), today.getMonth(), today.getDate());
                dayNextWeek.setDate(dayNextWeek.getDate() + 7);
                $scope.moveToDate(dayNextWeek);
                $accessibility.speak(getFullDateName(dayNextWeek));
            }

            $scope.moveToPreviousWeek = function () {
                var today = $scope.today;
                var dayNextWeek = new Date(today.getFullYear(), today.getMonth(), today.getDate());
                dayNextWeek.setDate(dayNextWeek.getDate() - 7);
                $scope.moveToDate(dayNextWeek);
                $accessibility.speak(getFullDateName(dayNextWeek));
            }

            $scope.moveToDate = function (dateToMoveTo) {
                $scope.today = new Date(dateToMoveTo.getFullYear(), dateToMoveTo.getMonth(), dateToMoveTo.getDate());
                getCardCalendarData();
                getCardCalendarMonth();

                var listControl = $scope.$ctrl.metadata;
                if (listControl.OnDayChanged) {
                    listControl.OnDayChanged($scope.today);
                }
            };

            $scope.$watch(function () { return $scope.$ctrl.data; }, function () {
                getCardCalendarData();
                getCardCalendarMonth();
            });
        },
        templateUrl: 'components/calendar/calendar.html',
    });
})();
