import { RemoteError as AppErrorCode } from "@ms-cds/data-contracts";

(function () {
    "use strict";
    angular
        .module("mobile-app.controllers")
        .controller("ChangeCompanyCtrl", function ($scope, $stateParams, $timeout, $label, axSession, dataServices, $pageModal, $accessibility, $filter) {
            $scope.$label = $label;

            $scope.openModal = function () {
                $scope.data = {
                    selectedCompany: {
                        companyId: ''
                    }
                };

                $scope.dataState = "loading";
                $scope.getAllCompaniesSupported = true;
                $scope.currentCompany = undefined;
                $scope.changeCompanyAllowed = true;
                $scope.companySaveNeeded = false;

                return $pageModal.fromTemplateUrl('components/company-selector/company-selector.html', {
                    scope: $scope,
                    hardwareBackButtonClose: true,
                    useDefaultStatusBar: true
                }).then(function (modal) {
                    $scope.changeCompanyModal = modal;
                    $scope.changeCompanyModal.show();
                    $scope.currentCompany = axSession.getCompany().toUpperCase();

                    return axSession.getCompatConfig().then(function (compatConfig) {
                        $scope.getAllCompaniesSupported = compatConfig.isGetAllCompaniesSupported;
                        loadCompanyData();
                    }, function (error) {
                        //Not able to get a session can be due to invalid company
                        if (error === AppErrorCode.InvalidCompany) {
                            $scope.getAllCompaniesSupported = false;
                            loadCompanyData();
                        } else {
                            errorHandler(error);
                        }
                    });
                });

            };

            $scope.$watch("data.selectedCompany.companyId", function () {
                if ($scope.data && $scope.data.selectedCompany && $scope.data.selectedCompany.companyId) {
                    let newCompanyId = $scope.data.selectedCompany.companyId.toUpperCase();
                    if ($scope.currentCompany && $scope.currentCompany !== newCompanyId) {
                        $scope.companySaveNeeded = true;
                    } else {
                        $scope.companySaveNeeded = false;
                    }
                }
            });

            $scope.saveSettings = function () {
                let newCompanyId = $scope.data.selectedCompany.companyId;
                axSession.changeCompany(newCompanyId);
                $scope.currentCompany = newCompanyId.toUpperCase();
                $scope.companySaveNeeded = false;
                $scope.closeChangeCompanyPage();
            };

            $scope.doRefresh = function () {
                if ($scope.dataState !== "loading" || $scope.dataState !== "refreshing") {
                    $scope.dataState = "refreshing";
                    loadCompanyData(0);
                }
            };

            $scope.searchCompanyTalkBack = function () {
                if ($accessibility && $accessibility.isScreenReaderOn()) {
                    const dataLength = ($filter('filter')($scope.data.companies, $scope.data.SearchByCompanyId)).length;
                    const talkBackString = $label("search_list").replace("%d", dataLength);
                    $accessibility.speak(talkBackString);
                }
            };

            $scope.isItemSelected = function (item) {
                return $scope.data.selectedCompany && $scope.data.selectedCompany.companyId === item.companyId;
            };

            function checkPendingSync() {
                if (dataServices.syncManager.getPendingSyncUnitCount() > 0) {
                    $scope.changeCompanyAllowed = false;
                }
                else {
                    $scope.changeCompanyAllowed = true;
                }
            }

            $scope.closeChangeCompanyPage = function () {
                let popOverObject = $scope.changeCompanyModal;
                if (popOverObject) {
                    popOverObject.hide().then(function () {
                        popOverObject.remove();
                    });
                }
            }

            // Gets a label with index and total amout of items for one single company selector's list item
            $scope.getAriaLabel = function (idx, filteredCompanies) {
                return `${idx + 1} ${$label("Of_label")} ${filteredCompanies.length}`;
            }

            function loadCompanyData(statelessness = $stateParams.allowedStaleness || (7 * 24 * 60 * 60)) {
                checkPendingSync();

                if ($scope.getAllCompaniesSupported) {
                    axSession.getAllCompanies(statelessness)
                        .then(applyCompanyList, function (error) {
                            errorHandler(error);
                        });
                } else {
                    //If getting all company is not supported set the data state as loaded
                    setDataStateToLoaded();

                    //Setting the current company on model
                    $scope.data = {
                        selectedCompany: {
                            companyId: $scope.currentCompany
                        }
                    };
                }
            }

            function errorHandler(error) {
                $scope.dataState = "error";
                $scope.errorCode = error;
                $scope.$apply();
            }

            function applyCompanyList(companyList) {
                var selectedCompanyId = getCurrentCompanyFromList(companyList);
                $scope.data = {
                    companies: companyList
                };
                //First apply the companies so that it renders list
                setDataStateToLoaded();
                //Now after list rendering is done, update model else in iOS it can not reflect the model selection properly
                $timeout(function () {
                    $scope.data.selectedCompany = {
                        companyId: selectedCompanyId
                    };
                });
            }

            function setDataStateToLoaded() {
                $scope.dataState = "loaded";
                $scope.$broadcast("scroll.refreshComplete");
                $scope.$apply();
            }

            function getCurrentCompanyFromList(companyList) {
                let currentCompanyFromList = companyList.find(function (item) {
                    return item.companyId.toUpperCase() === $scope.currentCompany;
                });

                return currentCompanyFromList.companyId;
            }
        });
})();
