import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import "./company-selector.js";
import { ngMock } from "./../../../src/ngMock";
import { asyncDone } from "../../../src/test-utils/asyncDone.spec";
import "../../js/PageModalService.ts";
import * as localization from "../../../src/localization/localization";

describe("Component: Company selector", function () {
    let companies = [
        {
            $hashKey: "obj1",
            $$hashKey: "obj1",
            companyId: "comp1",
            companyName: "Company 1",
        },
        {
            $hashKey: "obj2",
            $$hashKey: "obj2",
            companyId: "comp2",
            companyName: "Company 2",
        },
        {
            $hashKey: "obj3",
            $$hashKey: "obj3",
            companyId: "comp3",
            companyName: "Company 3",
        },
    ];
    beforeEach(ngMock.module("mobile-app", function ($provide, $injector) {
        $provide.provider("axSession", function () {
            this.$get = function () {
                return {
                    getCompany: function () {
                        return "comp2";
                    },
                    getAllCompanies: function () {
                        return Promise.resolve(companies);
                    },
                    getCompatConfig: function () {
                        return Promise.resolve({
                            isGetAllCompaniesSupported: true,
                        });
                    }
                };
            };
        });
    }));

    describe("with $ChangeCompanyCtrl", function () {
        let controller;
        let scope;
        let timeout;
        beforeEach(ngMock.inject(function ($rootScope, $stateParams, $timeout, $controller, $label, axSession, dataServices, $pageModal, $httpBackend) {
            $httpBackend.when("GET", "components/company-selector/company-selector.html").respond("");
            scope = $rootScope.$new();
            timeout = $timeout;
            controller = $controller("ChangeCompanyCtrl",
                {
                    $scope: scope,
                    $stateParams: $stateParams,
                    $label: $label,
                    axSession: axSession,
                    dataServices: dataServices,
                    $pageModal: $pageModal,
                }
            );
            scope.$apply = function () {
                return;
            };
        }));

        it("should isItemSelected return true for selected item", function () {
            let item = {
                companyId: "comp2",
            };
            return scope.openModal().then(function () {
                timeout.flush();
                expect(scope.isItemSelected(item)).to.be.equal(true);
            });
        });

        it("should aria-label with index be constructed correctly", function () {
            let idx = 0;
            let expectedLabel = `${idx + 1} ${localization.resolveLabel("Of_label")} ${companies.length}`;
            expect(scope.getAriaLabel(0, companies)).to.be.equal(expectedLabel);
        });
    });
});