import { getType } from "./rendererRegistration";
import * as Design from "../../../src/view-model/design";
import { ILayoutStyles } from "../../../src/view-model/design";
import { IControl } from "../../../src/view-model/control/BaseControl/IControl";
import * as angular from "angular";

export interface IControlScope extends angular.IScope {
    instance: IControl;
    control: IControl;
    isControlHidden(): boolean;
    getValue(): any;
    updateValue(value: any): void;
    getEntityRef(): string;
    metadata: any;
    data: any;
    isEditable: boolean;
    styles: ILayoutStyles;
    classes: string[];
}

let cachedCompile = new Map<string, angular.ITemplateLinkingFunction>();

angular.module("mobile-app").directive("control", function ($compile: angular.ICompileService, controlBinding) {
    return <angular.IDirective>{
        restrict: "E",
        replace: true,
        scope: {
            instance: "<",
        },
        link: function (scope: IControlScope, element, attrs) {
            let instance = scope.instance;
            let metadata = instance.metadata();
            let type = getType(instance);
            scope.isControlHidden = function () {
                return instance.hidden;
            };

            // Handle the ones which are not converted
            if (instance.generic) {
                scope.getValue = controlBinding.createBindingEvaluator("value");
                scope.updateValue = controlBinding.createBindingValueUpdater("value");
                scope.getEntityRef = controlBinding.createBindingEvaluator("entityRef");
                scope.metadata = metadata;
                scope.data = instance.dataContext();
                scope.isEditable = instance.isEditable();
                scope.styles = Design.getInlineLayoutStyles(instance.getDesign());
                scope.classes = Design.getControlStyleClasses(instance);
                let template: string;
                if (type === "timer") {
                    template = `<${type}
                        get-value="getValue(data, metadata.bindings)"
                        data="data"
                        entity-ref="getEntityRef(data, metadata.bindings)"
                        metadata="metadata"
                        is-editable="isEditable"
                        set-value="updateValue(data, metadata.bindings, newvalue)"
                        ng-style="styles"
                        ng-class ="classes"
                        class="control"
                        ng-if="!isControlHidden()"/>`;

                } else if (scope.metadata.Type === "List") {
                    instance.container = true;
                    scope.classes = Design.getControlStyleClasses(instance);
                    instance.getDataSource = controlBinding.getter("dataSource");
                    template = `<${type}
                        data="instance.getDataSource()"
                        metadata="metadata"
                        ng-style="styles"
                        ng-class ="classes"
                        class="control"
                        ng-if="!isControlHidden()"/>`;
                } else {
                    template = `<${type}
                        data="getValue(data, metadata.bindings)"
                        entity-ref="getEntityRef(data, metadata.bindings)"
                        metadata="metadata"
                        ng-style="styles"
                        ng-class ="classes"
                        class="control"
                        ng-if="!isControlHidden()"/>`;
                }

                element.replaceWith($compile(template)(scope));
            } else {
                // New controls only need the instance
                scope.control = instance;
                scope.styles = Design.getInlineLayoutStyles(instance.getDesign());
                scope.classes = Design.getControlStyleClasses(instance);

                let template = `<${type}
                    instance="control"
                    ng-style="styles"
                    ng-class="classes"
                    class="control"
                    ng-if="!isControlHidden()"/>`;

                createControl(type, template, $compile, scope, element);
            }
        }
    };
});

function createControl(type: string, template: string, $compile: angular.ICompileService, scope: angular.IScope, element) {
    let typeCompile = cachedCompile.get(type);
    if (!typeCompile) {
        typeCompile = $compile(template);
        cachedCompile.set(type, typeCompile);
    }

    return typeCompile(scope, clonedElement => element.replaceWith(clonedElement));
}