"use strict";
import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import "./example-component";
import {ngMock} from "./../../../src/ngMock";

describe("Component: myComponent", function () {
  beforeEach(ngMock.module("myComponentModule"));

  describe("with $compile", function () {
    let element;
    let scope;
    beforeEach(ngMock.inject(function($rootScope, $compile){
      scope = $rootScope.$new();
      element = angular.element('<my-component my-binding="{{outside}}"></my-component>');
      element = $compile(element)(scope);
      scope.outside = "1.5";
      scope.$apply();
    }));

    it("should render the text", function() {
      const h1 = element.find("h1");
      expect(h1.text()).to.be.equal("Unit Testing AngularJS 1.5");
    });

    it("should update the rendred text when the parent scope changes", function() {
      scope.outside = "2.0";
      scope.$apply();
      const h1 = element.find("h1");
      expect(h1.text()).to.be.equal("Unit Testing AngularJS 2.0");
    });

    describe("Controller: myComponent", function () {
      let controller;
      beforeEach(function() {
        controller = element.controller("myComponent");
      });

      it("should expose my title", function() {
        expect(controller.myTitle).to.not.be.undefined;
        expect(controller.myTitle).to.be.equal("Unit Testing AngularJS");
      });

      it("should have my binding bound", function() {
        expect(controller.myBinding).to.not.be.undefined;
        expect(controller.myBinding).to.be.equal("1.5");
      });
    });
  });

  describe("with $componentController", function () {
    let controller;
    let scope;
    beforeEach(ngMock.inject(function($rootScope, $componentController){
      scope = $rootScope.$new();
      controller = $componentController("myComponent", {$scope: scope}, {myBinding: "1.5"});
    }));

    it("should be attached to the scope", function() {
      expect(scope.$ctrl).to.be.equal(controller);
    });

    it("should expose my title", function() {
      expect(controller.myTitle).to.not.be.undefined;
      expect(controller.myTitle).to.be.equal("Unit Testing AngularJS");
    });

    it("should have my binding bound", function() {
      expect(controller.myBinding).to.not.be.undefined;
      expect(controller.myBinding).to.be.equal("1.5");
    });
  });
});
