import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import { ngMock } from "./../../../src/ngMock";
import { Field } from "../../../src/view-model/Control/Field/Field";
import * as controlBinding from "../../../src/view-model/controlBinding";
import "./field.js";

describe("Component: field", function () {
    let element;
    let scope;
    let elementScope;
    let instance;
    let dateVal;
    let mockInstanceProperties;

    beforeEach(() => {
        ngMock.module("mobile-app");
        mockInstanceProperties = {
            Id: "1",
            Name: "Control1",
            Label: "My Label",
            Type: "DateTime",
            bindings: {
                value: {
                    path: "myPath",
                }
            }
        };
        dateVal = "2017-02-26T19:59:48";
    });

    const compileElementWithScope = ngMock.inject(($rootScope, $compile) => {
        scope = $rootScope.$new();
        scope.instance = instance;
        element = angular.element('<field instance="instance"></field>');
        element = $compile(element)(scope);
        elementScope = element.children().scope();
        scope.$apply();
        return {
            element,
            elementScope
        };
    });

    describe("controller", function () {
        beforeEach(() => {
            instance = new Field(mockInstanceProperties, controlBinding);
            instance.dataBind({ myPath: dateVal });
            compileElementWithScope();
        });

        it("should attach the instance and services to the scope", ngMock.inject(($label, $navigate) => {
            expect(elementScope.$this).to.equal(instance);
            expect(elementScope.$label).to.equal($label);
            expect(elementScope.$navigate).to.equal($navigate);
        }));

        it("should render the correct label", function () {
            const h2 = element.find("h2");
            expect(h2.text()).to.be.equal("My Label");
        });

        it("should update the label when the metadata changes", function () {
            instance.metadata().Label = "New Label";
            scope.$apply();
            const h2 = element.find("h2");
            expect(h2.text()).to.be.equal("New Label");
        });
    });

    describe("when does not have reflink", function () {
        beforeEach(() => {
            instance = new Field(mockInstanceProperties, controlBinding);
            instance.dataBind({ myPath: dateVal });
            compileElementWithScope();
        });

        it("should render the correct formatted value in a p tag", function () {
            const p = element.find("p");
            expect(p.text()).to.be.equal(new Date(dateVal + "Z").toLocaleString());
        });
    });

    describe("when has reflink", function () {
        beforeEach(() => {
            mockInstanceProperties.ReferencePageId = "1";
            mockInstanceProperties.bindings.entityRef = {
                path: "myPath"
            };
            instance = new Field(mockInstanceProperties, controlBinding);
            instance.dataBind({ myPath: dateVal });
            compileElementWithScope();
        });

        it("should render the correct formatted value in a a tag", function () {
            const a = element.find("a");
            expect(a.text()).to.be.equal(new Date(dateVal + "Z").toLocaleString());
        });
    });

    describe("UnWrap Text", function () {
        it("should have class value-nowrap on a p tag when UnWrapText is set to true", function () {
            mockInstanceProperties.UnWrapText = true;
            instance = new Field(mockInstanceProperties, controlBinding);
            instance.dataBind({ myPath: dateVal });
            compileElementWithScope();
            const p = element.find("p");
            expect(p.hasClass("value-nowrap")).to.be.true;
        });

        it("should not have class value-nowrap on a p tag when UnWrapText is set to false", function () {
            mockInstanceProperties.UnWrapText = false;
            instance = new Field(mockInstanceProperties, controlBinding);
            instance.dataBind({ myPath: dateVal });
            compileElementWithScope();
            const p = element.find("p");
            expect(p.hasClass("value-nowrap")).to.be.false;
        });

        it("should not have class value-nowrap on a p tag when UnWrapText is snot set", function () {
            instance = new Field(mockInstanceProperties, controlBinding);
            instance.dataBind({ myPath: dateVal });
            compileElementWithScope();
            const p = element.find("p");
            expect(p.hasClass("value-nowrap")).to.be.false;
        });
    });
});
