﻿import { RemoteError } from "@ms-cds/data-contracts";
import { AccessibilityConstants } from "../../../www/components/accessibilityConstants";

const getAppErrorMessage = (code, label) => {
    switch (code) {
        case RemoteError.NetworkError:
            return label("Info-bar_NetworkError");
        case RemoteError.UnAuthenticated:
            return label("Info-bar_InvalidCredentials");
        case RemoteError.InvalidCompany:
            return label("Info-bar_InvalidCompany");
        case RemoteError.SeverConnectionFailed:
            return label("Info-bar_InvalidResponse");
        case RemoteError.Unknown:
            return label("Info-bar_UnknownError");
        case RemoteError.IndeterminateResult:
        case RemoteError.InvalidResponse:
            return label("Info-bar_NetworkError");
        case RemoteError.FileUploadError:
        default:
            return label("Info-bar_error");
    }
};


angular.module('mobile-app').directive('infoBar', function ($label, $accessibility, $interval) {
    var loginStarted = false;
    document.addEventListener("ax-loginComplete", onLoginFinished);
    document.addEventListener("ax-loginNeeded", onLoginStarted);

    function onLoginStarted() {
        loginStarted = true;
    };

    function onLoginFinished(event) {
        loginStarted = false;
    };

    return {
        restrict: 'E',
        replace: 'true',
        template: require("./info-bar.html"),
        scope: {
            state: '@',
            errorCode: '@'
        },
        link: function (scope, element, attrs) {
            scope.$label = $label;
            scope.getErrorMessage = function () {
                return getAppErrorMessage(scope.errorCode, scope.$label);
            };

            scope.getState = function () {
                //If a login has started, infobar should always shown the state instead of other states
                if (loginStarted === true) {
                    return "loginStarted";
                } else {
                    return scope.state;
                }
            };

            scope.getInfobarClassnames = function () {
                let infoState = scope.getState();

                switch (infoState) {
                    case 'loading':
                    case 'refreshing':
                    case 'loaded':
                    case 'loginStarted':
                        return "bar-positive";

                    case 'offline':
                        return 'bar-energized';

                    case 'error':
                        return 'bar-assertive';

                    default:
                        return '';
                }
            };
            scope.$on("$destroy", function() {
                clearInterval(scope.talkBackInterval);
            });
            if ($accessibility && $accessibility.isScreenReaderOn()) {
                scope.$watch(function () { return scope.state; }, function () {
                    var statusToSpeak;

                    //Login started should always be shown if the app is in a login flow
                    if (loginStarted === true) {
                        statusToSpeak = $label("Info-bar_LoginStarted");
                    } else if (scope.state == "loaded") {
                        statusToSpeak = $label("Info-bar_loaded");
                    } else if (scope.state == "offline") {
                        statusToSpeak = $label("Info-bar_offline");
                    } else if (scope.state == "error") {
                        statusToSpeak = scope.getErrorMessage();
                    } else if (scope.state == "loading") {
                        statusToSpeak = $label("Info-bar_loading_speak");
                    } else if (scope.state == "refreshing") {
                        statusToSpeak = $label("Info-bar_refreshing");
                    }
                    if (statusToSpeak) {
                        // Read status without interruption of current screen reader's speaking,
                        // as we need to wait for a title reading to complete
                        $accessibility.deferredSpeak(statusToSpeak);

                        function talkBackTimer() {
                            $accessibility.speak(statusToSpeak);
                        }
                        clearInterval(scope.talkBackInterval);
                        if (scope.state == "loading" || scope.state == "refreshing") {
                            scope.talkBackInterval = setInterval(function () {
                                talkBackTimer();
                            }, AccessibilityConstants.talkbackRepeatInterval);
                        } else {
                            scope.talkBackInterval = null;
                        }
                    }
                });
            };
        }
    };
});
