import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import "./info-bar.js";
import { ngMock } from "./../../../src/ngMock";
import { asyncDone } from "../../../src/test-utils/asyncDone.spec";
import { AccessibilityConstants } from "../../../www/components/accessibilityConstants";

describe("Directive: Info bar", function () {
    beforeEach(ngMock.module("mobile-app", function ($provide, $injector) {
        $provide.provider("$accessibility", function () {
            this.$get = function () {
                return {
                    isScreenReaderOn: function () {
                        return true;
                    },
                    speak: function () {
                        return;
                    },
                    deferredSpeak: function () {
                        return;
                    }
                };
            };
        });
    }));

    describe("Status speaking tests", function () {
        let originalTimeout;
        let controller;
        let scope;
        let timeoutService;
        let accessibilityService;
        let labelService;

        beforeEach(ngMock.inject(function ($rootScope, $compile, $accessibility, $label) {
            scope = $rootScope.$new();
            accessibilityService = $accessibility;
            labelService = $label;
            scope.dataState = "loaded";
            scope.errorCode = "";
            let element = angular.element('<info-bar state="{{dataState}}" error-code="{{errorCode}}"></info-bar>');
            element = $compile(element)(scope);
            scope.$digest();
        }));

        it("should info-bar read loading state", function () {
            let spokenStatus;

            accessibilityService.deferredSpeak = function (text) {
                spokenStatus = text;
            };

            scope.dataState = "loading";
            scope.$digest();

            expect(spokenStatus).to.be.equal(labelService("Info-bar_loading_speak"));
        });

        it("should info-bar read refreshing state", function () {
            let spokenStatus;

            accessibilityService.deferredSpeak = function (text) {
                spokenStatus = text;
            };

            scope.dataState = "refreshing";
            scope.$digest();

            expect(spokenStatus).to.be.equal(labelService("Info-bar_refreshing"));
        });

        it("should info-bar read error state", function () {
            let statusWasSpoken;

            accessibilityService.deferredSpeak = function (text) {
                statusWasSpoken = true;
            };

            scope.dataState = "error";
            scope.$digest();

            expect(statusWasSpoken).to.be.equal(true);
        });

        it("should info-bar read offline state", function () {
            let spokenStatus;

            accessibilityService.deferredSpeak = function (text) {
                spokenStatus = text;
            };

            scope.dataState = "offline";
            scope.$digest();

            expect(spokenStatus).to.be.equal(labelService("Info-bar_offline"));
        });

        it("should info-bar read loaded state", function () {
            scope.dataState = "loading";
            scope.$digest();

            let spokenStatus;

            accessibilityService.deferredSpeak = function (text) {
                spokenStatus = text;
            };

            scope.dataState = "loaded";
            scope.$digest();

            expect(spokenStatus).to.be.equal(labelService("Info-bar_loaded"));
        });
    });
});