import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import { ngMock } from "./../../../src/ngMock";
import { List } from "../../../src/view-model/Control/List/List";
import { IListDesign, IListMetadata } from "../../../src/view-model/Control/List/IList";
import { IControlMetadata } from "../../../src/view-model/Control/BaseControl/IControl";
import { IControlMetadataExtension } from "../../../src/view-model/Control/BaseControl/Control";
import { IPageOptions } from "../../../src/view-model/IPage";
import { Page, IPageConfig } from "../../../src/view-model/page";
import * as telemetry from "../../../src/services/telemetry";

import * as controlBinding from "../../../src/view-model/controlBinding";
import "./list.js";

describe("Component: list", function () {
    let element;
    let scope;
    let elementScope;
    let instance: List;
    let page;
    let mockInstanceProperties;
    let listMetadataChildren: (IControlMetadata & IControlMetadataExtension)[] = [
        {
            Type: "Input",
            Label: "TestVisibleInputOne",
            Id: "1",
            bindings: { value: { path: "controlPath" } },
        },
    ];
    const compileElementWithScope = ngMock.inject(($rootScope, $compile) => {
        scope = $rootScope.$new();
        scope.instance = instance;
        element = angular.element('<list instance="instance"></list>');
        element = $compile(element)(scope);
        elementScope = element.children().scope();
        scope.$apply();
        return {
            element,
            elementScope
        };
    });
    beforeEach(() => {
        ngMock.module("mobile-app");
        mockInstanceProperties = {
        };
    });

    describe("List Items", function () {
        beforeEach(() => {
            const options: IPageOptions = {
                appId: "test app Id",
                pageId: "test Page Id",
            };
            const mockMetadataServiceFull = {
                getPageMetadata: function () {
                    return Promise.resolve({
                        Title: "My Test form",
                        colorName: "test-color",
                        RecordingId: "6f85b4f0-16c7-4668-b7e4-0a518996853b",
                        SyncMode: 0,
                        query: undefined,
                        Controls: [
                            {
                                BoundEntity: "FMRental",
                                BoundProperty: "RentalId",
                                FilterPrefix: "FMMobReservationDetails:FMRental:RentalId",
                                HelpText: null,
                                Id: "b0ae5482-61dd-46ef-a45e-ec78f29ad2b5",
                                Label: "Vehicle rental ID",
                                Name: "FMRental_RentalId",
                                SyncMode: 2,
                                Type: "Input",
                                bindings: {
                                    value: {
                                        path: "b0ae5482-61dd-46ef-a45e-ec78f29ad2b5"
                                    }
                                },
                            },
                            {
                                $type: "ReferenceControlMetadata",
                                ReferenceEntity: "FMCustomer",
                                ReferencePageId: "986D5DB6-96D9-419D-BED0-9CAF54439B03",
                                Id: "0b6dc301-6a63-44f7-ba90-138feb2bc97c",
                                Type: "Input",
                                Label: "Customer",
                                Name: "FMCustomer_FullName",
                                HelpText: null,
                                SyncMode: 2,
                                BoundEntity: "FMRental",
                                BoundProperty: "FMCustomer/fullName",
                                FilterPrefix: null,
                                bindings: {
                                    value: {
                                        path: "0b6dc301-6a63-44f7-ba90-138feb2bc97c.value"
                                    },
                                    entityRef: {
                                        path: "0b6dc301-6a63-44f7-ba90-138feb2bc97c.$ref"
                                    }
                                },
                                ExtType: "Reference",
                                ReferenceAppId: "B6EDC94F-F10E-4050-B79E-B56C96536816"
                            },
                            {
                                $type: "ReferenceControlMetadata",
                                ReferenceEntity: "FMVehicle",
                                ReferencePageId: null,
                                Id: "f2cc723d-0a96-40e1-a8b3-2cb52fc09c80",
                                Type: "Integer",
                                Label: "Vehicle mileage",
                                Name: "FMVehicle_Mileage",
                                HelpText: null,
                                SyncMode: 2,
                                BoundEntity: "FMRental",
                                BoundProperty: "FMVehicle/Mileage",
                                FilterPrefix: null,
                                bindings: {
                                    value: {
                                        path: "f2cc723d-0a96-40e1-a8b3-2cb52fc09c80.value"
                                    },
                                    entityRef: {
                                        path: "f2cc723d-0a96-40e1-a8b3-2cb52fc09c80.$ref"
                                    }
                                },
                                ExtType: "Reference",
                                ReferenceAppId: "B6EDC94F-F10E-4050-B79E-B56C96536816"
                            },
                            {
                                $type: "EnumControlMetadata",
                                EnumName: "FMReservationState",
                                Id: "c91c788d-5f77-4eb6-9312-473fb514fc7a",
                                Type: "ComboBox",
                                Label: "Status",
                                Name: "FMRental_State",
                                HelpText: null,
                                SyncMode: 2,
                                BoundEntity: "FMRental",
                                BoundProperty: "State",
                                FilterPrefix: "FMMobReservationDetails:FMRental:State",
                                bindings: {
                                    value: {
                                        path: "c91c788d-5f77-4eb6-9312-473fb514fc7a"
                                    },
                                    options: [
                                        {
                                            text: "New",
                                            value: 0
                                        },
                                        {
                                            text: "Ready for pickup",
                                            value: 1
                                        },
                                        {
                                            text: "In progress",
                                            value: 2
                                        },
                                        {
                                            text: "Complete",
                                            value: 3
                                        }
                                    ]
                                }
                            },
                            {
                                Id: "9a442ed9-6cdb-4fe6-8da7-fb299882bb75",
                                Type: "DateTime",
                                Label: "Return",
                                Name: "FMRental_EndDate",
                                HelpText: null,
                                SyncMode: 2,
                                BoundEntity: "FMRental",
                                BoundProperty: "EndDate",
                                FilterPrefix: "FMMobReservationDetails:FMRental:EndDate",
                                bindings: {
                                    value: {
                                        path: "9a442ed9-6cdb-4fe6-8da7-fb299882bb75"
                                    }
                                }
                            },
                            {
                                Id: "e9f5ed41-0d78-4e55-a703-7c9dabcd84bf",
                                Type: "Real",
                                Label: "Rate per day",
                                Name: "FMRental_VehicleRatePerDay",
                                HelpText: null,
                                SyncMode: 2,
                                BoundEntity: "FMRental",
                                BoundProperty: "VehicleRatePerDay",
                                FilterPrefix: "FMMobReservationDetails:FMRental:VehicleRatePerDay",
                                bindings: {
                                    value: {
                                        path: "e9f5ed41-0d78-4e55-a703-7c9dabcd84bf"
                                    }
                                }
                            },
                            {
                                $type: "ReferenceControlMetadata",
                                ReferenceEntity: "FMCustomer",
                                ReferencePageId: null,
                                Id: "e63e12c7-c8a5-49b9-b82b-6cda72e7928c",
                                Type: "Image",
                                Label: "Photo",
                                Name: "FMCustomer_Image",
                                HelpText: null,
                                SyncMode: 2,
                                BoundEntity: "FMRental",
                                BoundProperty: "FMCustomer/Image",
                                FilterPrefix: null,
                                bindings: {
                                    value: {
                                        path: "e63e12c7-c8a5-49b9-b82b-6cda72e7928c.value"
                                    },
                                    entityRef: {
                                        path: "e63e12c7-c8a5-49b9-b82b-6cda72e7928c.$ref"
                                    }
                                },
                                ExtType: "Reference",
                                ReferenceAppId: "B6EDC94F-F10E-4050-B79E-B56C96536816"
                            },
                        ],
                        Tasks: [],
                        Design: {
                            items: [
                                {
                                    _name: "Fields",
                                    items: [
                                        { name: "FMRental_RentalId" },
                                        { name: "FMCustomer_FullName" },
                                        { name: "FMVehicle_Mileage" },
                                        { name: "FMRental_State" },
                                        { name: "FMRental_EndDate" },
                                        { name: "FMRental_VehicleRatePerDay" },
                                        { name: "FMCustomer_Image" },
                                    ],
                                }
                            ]
                        }
                    });
                }
            };
            const mockDataServiceNonNull: any = {
                readData: () => {
                    return Promise.resolve({
                        data: {},
                    });
                }
            };
            let config: IPageConfig = {
                $dataService: mockDataServiceNonNull,
                $metadataService: mockMetadataServiceFull,
                $telemetry: telemetry,
                $businessLogic: undefined,
            };
            page = new Page(options, config);

        });

        afterEach(() => {
            page = undefined;
            instance = undefined;
        });

        it("should use default design if a customized design is not used", function () {
            instance = new List({ Children: listMetadataChildren, bindings: { dataSource: { path: "listDataPath" } } }, controlBinding, page);
            instance.dataBind({
                listDataPath: [
                    {
                        CanonicalId: "1",
                        controlPath: "row 1",
                    },
                ]
            });
            compileElementWithScope();
            let firstRow: HTMLElement = element[0].querySelector(".list-item");
            expect(firstRow.classList.contains("list-item-is-custom")).to.be.false;
        });
        it("should ignore default design if a customized design is used", function () {
            instance = new List({ Children: listMetadataChildren, bindings: { dataSource: { path: "listDataPath" } } }, controlBinding, page);
            instance.dataBind({
                listDataPath: [
                    {
                        CanonicalId: "1",
                        controlPath: "row 1",
                    },
                ]
            });
            const design = { padding: "none", design: { border: "none" } } as IListDesign;
            instance.applyDesign(design);
            compileElementWithScope();
            let firstRow: HTMLElement = element[0].querySelector(".list-item");
            expect(firstRow.classList.contains("list-item-is-custom")).to.be.true;
        });
        describe("showEmptyMessage", () => {
            it("should return true if the list is empty and hideEmptyListMessage is false", () => {
                instance = new List({ Children: listMetadataChildren, bindings: { dataSource: { path: "listDataPath" } } }, controlBinding, page);
                instance.dataBind({
                    listDataPath: [
                    ]
                });
                compileElementWithScope();
                expect(elementScope.showEmptyMessage()).to.be.true;
            });
            it("should return false if the list is not empty and hideEmptyListMessage is false", () => {
                instance = new List({ Children: listMetadataChildren, bindings: { dataSource: { path: "listDataPath" } } }, controlBinding, page);
                instance.dataBind({
                    listDataPath: [
                        {
                            CanonicalId: "1",
                            controlPath: "row 1",
                        },
                    ]
                });
                compileElementWithScope();
                expect(elementScope.showEmptyMessage()).to.be.false;
            });
            it("should return false if the list is empty and hideEmptyListMessage is true", () => {
                instance = new List({ Children: listMetadataChildren, bindings: { dataSource: { path: "listDataPath" } } }, controlBinding, page);
                instance.dataBind({
                    listDataPath: [
                    ]
                });
                instance.metadata = () => {
                    return {
                        HideEmptyListMessage: true
                    };
                };
                compileElementWithScope();
                expect(elementScope.showEmptyMessage()).to.be.false;
            });
            it("should return false if the list is not empty and hideEmptyListMessage is true", () => {
                instance = new List({ Children: listMetadataChildren, bindings: { dataSource: { path: "listDataPath" } } }, controlBinding, page);
                instance.dataBind({
                    listDataPath: [
                        {
                            CanonicalId: "1",
                            controlPath: "row 1",
                        },
                    ]
                });
                instance.metadata = () => {
                    return {
                        HideEmptyListMessage: true
                    };
                };
                compileElementWithScope();
                expect(elementScope.showEmptyMessage()).to.be.false;
            });
        });
    });
});