﻿(function () {
    'use strict';

    angular.module('mobile-app.controllers').controller('NotificationsHubCtrl', function ($scope, $stateParams, $timeout, notificationServices, dataServices, telemetry, EntityRef, $label) {
        $scope.notificationList = [];
        $scope.dataState = "loading";
        $scope.$label = $label;

        var notificationListener = function () {
            $timeout(function () {
                applyNotificationList();
            });
        };

        var entityRef = EntityRef.parse($stateParams && $stateParams.entityContext);
        var status = $stateParams && $stateParams.status;

        function applyNotificationList() {
            $scope.notificationList = GetNotificationProperties();
            $scope.dataState = "loaded";
            telemetry.trackEvent(telemetry.events.notificationCenterLoaded.Name);
        }

        function errorHandler(error) {
            $scope.dataState = "error";
            console.log(error);
        }

        var GetNotificationProperties = function () {
            var notifications = [];
            if (entityRef && entityRef.id) {
                // If we have an entity context, only show changes for this entity
                notifications = notificationServices.getNotifications("$sync-" + entityRef.id);
                notifications = notifications.filter(function (item) {
                    return (status != "pending" || !item.canDelete);
                });
            } else {
                notifications = notificationServices.getNotifications();
            }
            return notifications;
        }

        $scope.doRefresh = function () {
            applyNotificationList();
            dataServices.syncManager.startProcessSyncQueue();
            $scope.$broadcast('scroll.refreshComplete');
        };

        $scope.invokeAction = function (actionName, notificationId) {
            notificationServices.invokeAction(actionName, notificationId);
        }

        $scope.$on("$ionicView.enter", function (event, data) {
            notificationServices.addListener(notificationListener);
        });

        $scope.$on("$ionicView.leave", function (event, data) {
            notificationServices.removeListener(notificationListener);
        });

        let notificationExpanded = {};
        $scope.toggleExpanded = function (notification) {
            if (notificationExpanded[notification.id] === undefined) {
                notificationExpanded[notification.id] = false;
            }
            notificationExpanded[notification.id] = !notificationExpanded[notification.id];
        };

        $scope.isExpanded = function (notification) {
            return notificationExpanded[notification.id] || false;
        };

        applyNotificationList();
    });
})();
