import { enableDataCaching } from "../device-security-check/device-security-check";
import { AccessibilityConstants } from "../accessibilityConstants";
import * as localization from "../../../src/localization";

angular.module('mobile-app.controllers').controller('ShellCtrl', function ($q, $scope, $http, $ionicModal, $ionicPopup, $ionicLoading, $cordovaInAppBrowser, $rootScope, $state, $timeout, $ionicHistory, axSession, notificationServices, networkStatus, telemetry, dataServices, SyncUnitStatus, $label, $ionicSideMenuDelegate, cacheManager, deviceSecurityCheck, $accessibility) {

    // TODO: Import the enum when converting to a .ts file
    const securityCheckResult = {
        notSecure: 0,
        secure: 1,
        indeterminate: 2,
        error: 3
    };

    // ensure security check on app resume
    document.addEventListener("resume", () => {
        if (axSession.isLoggedIn()) {
            let message = $label("Device_Passcode_Setup_Prompt_Body_Extended");
            deviceSecurityCheck.checkSecurity(true, message);
        }
    }, true);

    $scope.notifications = notificationServices.getNotifications();

    $scope.isSideMenuOpen = $ionicSideMenuDelegate.isOpen.bind($ionicSideMenuDelegate);

    // Form data for the login modal
    $scope.loginData = {
        server: axSession.getAosUrl(),
        company: axSession.getCompany(),
        urlArray: []
    };

    $scope.doesRecentLoginHasValue = function () {
        $scope.getAosUrlArray();
        return $scope.loginData.urlArray && $scope.loginData.urlArray.length > 0;
    };

    $scope.getAosUrlArray = function () {
        var currentUrlArray = $scope.loginData.urlArray;
        var updatedUrlArray = axSession.getAosUrlArray();
        for (var i = 0; i < updatedUrlArray.length; i++) {
            if (currentUrlArray.indexOf(updatedUrlArray[i]) == -1) {
                currentUrlArray.push(updatedUrlArray[i]);
            }
        }
        $scope.loginData.urlArray = currentUrlArray;
        return $scope.loginData.urlArray;
    };

    $scope.loggingInSpeaking = function () {
        $accessibility.speak($label("Info-bar_LoginStarted"));
        return setInterval(function () {
            $accessibility.speak($label("Info-bar_LoginStarted"))
        }, AccessibilityConstants.talkbackRepeatInterval);
    };

    $scope.userName = axSession.getUserName();
    $scope.userEmail = axSession.getUserEmail();
    $scope.company = axSession.getCompany();
    $scope.serverURL = axSession.getAosUrl();

    $scope.SLT = "";
    $scope.TPN = "";

    var legalUrls = {
        android: {
            "SLT": "https://go.microsoft.com/fwlink/?linkid=826585",
            "TPN": "https://go.microsoft.com/fwlink/?linkid=834558",
        },
        iOS: {
            "SLT": "https://go.microsoft.com/fwlink/?linkid=834563",
            "TPN": "https://go.microsoft.com/fwlink/?linkid=834564",
        },
        windows: {
            "SLT": "https://go.microsoft.com/fwlink/?linkid=834565",
            "TPN": "https://go.microsoft.com/fwlink/?linkid=834566",
        }
    };

    $scope.$label = $label;
    $scope.showAbout = function () {
        $ionicModal.fromTemplateUrl('components/about/about.html', {
            hardwareBackButtonClose: true,
            scope: $scope,
        })
            .then(function (aboutModal) {
                $scope.aboutModal = aboutModal;
                aboutModal.show();
            });
    };

    $scope.version = process.env.MOBILE_APP_VERSION;

    $scope.openSLT = function () {
        var url = "https://lcs.dynamics.com/Logon/Legal";
        switch (device.platform) {
            case "Android":
                url = legalUrls.android.SLT;
                break;
            case "iOs":
                url = legalUrls.ios.SLT;
                break;
            case "Windows":
                url = legalUrls.windows.SLT;
                break;
        }

        window.open(url, '_system');
        return false;
    };

    $scope.openTPN = function () {
        var url = "https://lcs.dynamics.com/Logon/Legal";
        switch (device.platform) {
            case "Android":
                url = legalUrls.android.TPN;
                break;
            case "iOs":
                url = legalUrls.ios.TPN;
                break;
            case "Windows":
                url = legalUrls.windows.TPN;
                break;
        }

        window.open(url, '_system');
        return false;
    };

    $scope.formatVersion = function (v) {
        var dotString = ` ${$label("Dot_label")} `;
        var version = v.replace(/\./g, dotString);

        return `${$label('Product_version_text')} ${version}`;
    }

    $scope.openLCS = function () {
        var url = "https://lcs.dynamics.com/Logon/Index";
        window.open(url, '_system');
        return false;
    };

    $scope.shouldShowFRCompliance = localization.getLanguage() === "fr";

    document.addEventListener("ax-userDataUpdated", function (event) {
        var data = event.detail;
        $scope.userName = data.userName;
        $scope.userEmail = data.userEmail;
        $scope.company = data.company;
        $scope.serverURL = $scope.loginData.server;
    });

    // Turn on debug functionality if not running under cordova.
    $scope.isInDebug = !window.cordova;

    var handleLaunchError = function () {
        //Only show the message when we the user is initiating the login flow
        // and not when login is triggered silently to refresh the token
        if ($scope.attemptingLogin === true) {
            $accessibility.speak($label("Launch_Error_Message"));
            $ionicPopup.alert({ title: $label("Launch_Error_Message") });
            $scope.attemptingLogin = false;
        }
        onLoginComplete(false);
    };

    // Create the login modal that we will use later
    var loginModalPromise = $ionicModal.fromTemplateUrl('components/login/login.html', {
        scope: $scope,
        backdropClickToClose: false,
        hardwareBackButtonClose: false
    }).then(function (modal) {
        $scope.modal = modal;
    });

    // Open the login modal
    var showLoginModal = function () {
        loginModalPromise.then(function () {
            $scope.modal.show();
            telemetry.trackEvent(telemetry.events.loginShown.Name);
        }).catch(handleLaunchError);
    };

    // Shows the login connect confirmation prompt to the user
    // if the user accespts, resolves the promise with "true"
    var showLoginConnectConfirmationPrompt = function () {
        var response = $q.defer();
        $scope.loginData.hasConfirmConnect = false;
        var warningMsg = $label("Login_connect_prompt_part1") + $scope.loginData.server + "<br>" + $label("Login_connect_prompt_part2");
        $accessibility.speak(warningMsg);
        $ionicPopup.show({
            title: warningMsg,
            content: '<div role="checkbox" aria-checked="{{loginData.hasConfirmConnect}}" aria-label="{{$label(\'Login_connect_prompt_confirm\')}}">' +
                '<ion-checkbox ng-model="loginData.hasConfirmConnect" ng-checked="loginData.hasConfirmConnect" aria-hidden="true">{{$label("Login_connect_prompt_confirm")}}</ion-checkbox></div>',
            scope: $scope,
            cssClass: 'login-confirm-popup',
            buttons: [
                {
                    text: $label("Actions_cancel"),
                    onTap: function (e) {
                        response.resolve(false);
                    }
                },
                {
                    text: $label("Login_button_text"),
                    type: 'button-positive',
                    onTap: function (e) {
                        if (!$scope.loginData.hasConfirmConnect) {
                            e.preventDefault();
                        }
                        else {
                            console.log("user confirmed connection to aos instance.");
                            response.resolve(true);
                        }
                    }
                }],
        });

        return response.promise;
    };

    var showLoginInAppBrowser = function () {
        let loggingInTimeInterval = $scope.loggingInSpeaking();
        // Show the login browser
        var options = {
            hidden: 'yes',
            location: 'no',
            toolbar: 'no',
            fullscreen: 'yes',
            hardwareback: 'no'
        };

        var loginURL = $scope.loginData.server + '/applanding.htm?_=' + Date.now();

        if (window.cordova) {
            var clearShowTimeout = function () {
                if (inAppBrowserShowTimeout) {
                    clearTimeout(inAppBrowserShowTimeout);
                    inAppBrowserShowTimeout = undefined;
                }
            };
            var unsubscribeEvents = function () {
                clearShowTimeout();
                if (offLoadStop) offLoadStop(); offLoadStop = undefined;
                if (offLoadError) offLoadError(); offLoadError = undefined;
                if (offExit) offExit(); offExit = undefined;
            };
            var inAppBrowserShowTimeout;
            // Monitor the pages being loaded
            var offLoadStop = $rootScope.$on('$cordovaInAppBrowser:loadstop', function (e, event) {
                clearTimeout(loggingInTimeInterval);
                if (event.url && event.url.toLowerCase().indexOf($scope.loginData.server.toLowerCase()) === 0) {
                    clearShowTimeout();
                    // Execute the in-app browser script to get the CSRF token
                    $http({
                        method: 'GET',
                        url: 'js/axLogin-getCsrfToken.js',
                    })
                        .then(function onSuccess(response) {
                            $cordovaInAppBrowser
                                .executeScript({ code: response.data })
                                .then(function handleCsrfToken(csrfToken) {
                                    axSession.setCsrfToken(csrfToken && csrfToken[0]);
                                    // We loaded our target marker page and we are logged in, so we are now logged in
                                    onLoginComplete(true);
                                    unsubscribeEvents();
                                    $cordovaInAppBrowser.close();
                                }).catch(handleLaunchError);
                        });
                }
                else {
                    if (!inAppBrowserShowTimeout) {
                        inAppBrowserShowTimeout = window.setTimeout(function () {
                            // Some other page loaded, show the in-app browser which was loaded hidden initially
                            $cordovaInAppBrowser.show();
                            inAppBrowserShowTimeout = undefined;

                        }, 1000);
                    }
                }
            });
            // If an error happens loading the in-app browser, hide it and cancel login flow
            var offLoadError = $rootScope.$on('$cordovaInAppBrowser:loaderror', function (e, event) {
                unsubscribeEvents();
                handleLaunchError();
                $cordovaInAppBrowser.close();
            });
            // If we are exiting the login-flow in in-app browser, unsubscribe events
            var offExit = $rootScope.$on('$cordovaInAppBrowser:exit', function (e, event) {
                unsubscribeEvents();
                $scope.attemptingLogin = false;
            });
        }
        else {
            // Do we have a debug CSRF token in the cache?
            var csrfData = cacheManager.getData("debug-csrf");
            if (csrfData && csrfData.timestamp > (Date.now() - (30 * 60 * 1000))) {
                // Assume CSRF is valid for 30 minutes
                axSession.setCsrfToken(csrfData.token);
                onLoginComplete(true);
                loginURL = null;
            }
            else {
                loginURL = loginURL + '&client=web';
                $ionicPopup.prompt({
                    title: $label("Debug_login_prompt"),
                    template: $label("Debug_csrf_prompt")
                }).then(function (csrfToken) {
                    if (csrfToken) {
                        console.log('csrf token is:', csrfToken);
                        cacheManager.setData("debug-csrf", { timestamp: Date.now(), token: csrfToken });
                        axSession.setCsrfToken(csrfToken);
                        onLoginComplete(true);
                    }
                    else {
                        onLoginComplete(false);
                    }
                }).catch(handleLaunchError);
            }
        }
        if (loginURL) {
            // Show the login window
            $cordovaInAppBrowser.open(loginURL, '_blank', options);
            telemetry.trackEvent(telemetry.events.loginShown.Name);
        }
    };

    var closeLoginModal = function () {
        if ($scope.modal) {
            $scope.modal.hide();
        }
    };

    var signalLoginComplete = function (success) {
        if (success) {
            telemetry.trackEvent(telemetry.events.loginSuccessful.Name);
        } else {
            telemetry.trackEvent(telemetry.events.loginUnsuccessful.Name);
        }
        document.dispatchEvent(new CustomEvent("ax-loginComplete", { detail: { success: success } }));
    }

    /**
     * Called after user accepts disclaimer during login flow to set the current aos url
     */
    var configureAxService = function () {
        var currentAOSUrl = axSession.getAosUrl();
        if (currentAOSUrl && currentAOSUrl !== $scope.loginData.server) {
            //if the user have connected to a new aos server, then clear previous company
            //Company will now set to the default company for the user
            axSession.setCompany("");
        }
        axSession.setAosUrl($scope.loginData.server);
    };

    $scope.Syncing = false;
    $scope.Online = true;
    $scope.pendingSyncCount = 0;
    $scope.SyncStatus;
    $scope.SyncStatusDescription = "";
    $scope.refreshSyncStatus = function () {
        $scope.SyncStatusDescription = $label("Notifications_label") + '.';
        if ($scope.errorNotificationCount > 0 && !$scope.Syncing) {
            $scope.SyncStatus = "syncError";
            $scope.SyncStatusDescription += ' ' + $label("Sync_status_error");
        }
        else if ($scope.Online) {
            $scope.SyncStatus = "syncOn";
            $scope.SyncStatusDescription += ' ' + $scope.pendingSyncCount;
            $scope.SyncStatusDescription += ' ' + $label("Sync_status_pending");
            if ($scope.Syncing) {
                $scope.SyncStatusDescription += ' ' + $label("Sync_status_syncing");
            }

        } else {
            $scope.SyncStatus = "syncOff";
            $scope.SyncStatusDescription += ' ' + $scope.pendingSyncCount;
            $scope.SyncStatusDescription += ' ' + $label("Sync_status_offline");
        }
    }

    function networkChanged(online) {
        $timeout(function () {
            $scope.Online = online;
            $scope.refreshSyncStatus();
        });
    }

    networkStatus.addOfflineListener(networkChanged.bind(null, false));
    networkStatus.addOnlineListener(networkChanged.bind(null, true));

    function onResume() {
        var online = navigator.connection.type !== Connection.NONE;
        networkChanged(online);
    }

    document.addEventListener("resume", onResume, false);

    $scope.refreshSyncStatus();

    var setNotificationBadge = function (badgeCount) {
        if (window.cordova) {
            if (badgeCount && badgeCount > 0) {
                window.cordova.plugins.notification.badge.set($scope.pendingSyncCount);
            } else {
                window.cordova.plugins.notification.badge.clear();
            }
        }
    }

    var setPendingSyncCount = function () {
        $scope.pendingSyncCount = notificationServices.getNotificationCount('$sync');
        setNotificationBadge($scope.pendingSyncCount);
    }


    // begin syncManager status
    let lastSyncState;
    dataServices.syncManager.subscribeStateChanged(function syncManagerStateChanged(newState) {
        lastSyncState = newState;
        $timeout(function () {
            $scope.Syncing = $rootScope.syncOn = (lastSyncState === 'Processing');
            setPendingSyncCount();
            $scope.refreshSyncStatus();
        });
    });

    setPendingSyncCount();
    dataServices.syncManager.subscribeSynUnitQueued(function syncUnitPending(syncUnits) {
        $timeout(function () {
            setPendingSyncCount();
        });
    });
    // end syncManager

    // notificationServices
    var getNotificationErrorCount = function () {
        $scope.errorNotificationCount = 0;
        var notifications = notificationServices.getNotifications();
        if (notifications) {
            for (var i = 0; i < notifications.length; i++) {
                if (notifications[i].priority >= notificationServices.notificationPriority.high) {
                    $scope.errorNotificationCount++;
                }
            }

        }
    };
    getNotificationErrorCount(); // run on startup;

    notificationServices.addListener(function () {
        $timeout(function () {
            getNotificationErrorCount();
            $scope.refreshSyncStatus();
        });
    });
    // end notificationServices

    var onLoginComplete = function (success) {
        $scope.attemptingLogin = false;
        signalLoginComplete(success);
        if (success) {
            // logged in
            closeLoginModal();
            setPendingSyncCount();
        }
    }

    var verifyMetadataAvailable = function () {
        var validationUrl = $scope.loginData.server + '/metadata';
        return $http({
            method: 'GET',
            url: validationUrl
        })
    }
    // Perform the login action when the user submits the login form
    var doLogin = $scope.doLogin = function doLogin() {
        let loggingInTimeInterval = $scope.loggingInSpeaking();

        if ($scope.attemptingLogin) {
            return;
        }

        if (!$scope.loginData.server) {
            return;
        }

        // Verify that the URL starts with https://
        if ($scope.loginData.server.toLowerCase().indexOf("https://") !== 0) {
            $accessibility.speak($label("Login_URL_https"));
            $ionicPopup.alert({ title: $label("Login_URL_https") });
            return;
        }

        //Remove all trailing slash from login server url
        var loginServerUrl = $scope.loginData.server;
        while (loginServerUrl.length > 0 && loginServerUrl[loginServerUrl.length - 1] == '/') {
            loginServerUrl = loginServerUrl.substring(0, loginServerUrl.length - 1);
        }
        $scope.loginData.server = loginServerUrl;

        $scope.attemptingLogin = true;
        // enable check for device security
        return deviceSecurityCheck.checkSecurity(true)
            .then((result) => {
                if (result === securityCheckResult.secure) {
                    return enableDataCaching();
                }
            })
            .then(() => {
                $scope.attemptingLogin = true;
                // Verify that the URL provided is an AX url, make a get request for metadata which doesn't require auth
                verifyMetadataAvailable()
                    .then(function onSuccess(response) {
                        clearInterval(loggingInTimeInterval);
                        // Check for AX headers to validate the server further
                        if (!response.headers('ms-dyn-aid')) {
                            $scope.attemptingLogin = false;
                            // If the dyn-aid header doesn't exist, it is not a valid AX server
                            $accessibility.speak($label("Login_URL_invalid"));
                            $ionicPopup.alert({ title: $label("Login_URL_invalid") });
                            return;
                        }

                        // Show prompt before connecting
                        showLoginConnectConfirmationPrompt().then(function (accepted) {
                            if (accepted) {
                                // Save user entry for AX
                                configureAxService();

                                // Show the hosted login flow
                                showLoginInAppBrowser();
                            }
                            else {
                                $scope.attemptingLogin = false;
                            }
                        }).catch(handleLaunchError);
                    },
                    function error(response) {
                        clearInterval(loggingInTimeInterval);
                        $scope.attemptingLogin = false;
                        if (response && response.status === 404) {
                            // Not found, this is an invalid URL
                            $accessibility.speak($label("Login_URL_invalid"));
                            $ionicPopup.alert({ title: $label("Login_URL_invalid") });
                            return;
                        }
                        $accessibility.speak($label("Login_URL_error"));
                        $ionicPopup.alert({ title: $label("Login_URL_error") });
                        return;
                    });
            }, (error) => {
                clearInterval(loggingInTimeInterval);
                $scope.attemptingLogin = false;
            });
    };

    document.addEventListener("ax-loginNeeded", function (evt) {
        if (evt.detail.softRetry) {
            // To warmup network do a call to get metadata and proceed to inAppBrowser even if metadata call fails
            verifyMetadataAvailable()
                .then(showLoginInAppBrowser)
                .catch(showLoginInAppBrowser);
        }
        else {
            showLoginModal();
        }
    });

    $rootScope.logOut = function () {
        var loggedOut = false;
        var completeLogout = function () {
            if (loggedOut) return;
            loggedOut = true;
            $ionicLoading.hide();
            // Clear out any cached stuff and then redirect to the applist.
            $ionicHistory.clearCache();
            $ionicHistory.clearHistory();
            //setting notification badge to 0 will clear the badge count
            setNotificationBadge(0);
            axSession.clearLogin(true);
            telemetry.trackEvent(telemetry.events.logoutSuccessful.Name);
            $state.go('app.appList', { allowedStaleness: 0 });
        };

        // Show a modal while we are logging out
        var showPromise = $ionicLoading.show({
            template: $label("Logout_modal"),
        });

        telemetry.trackEvent(telemetry.events.logout.Name);

        // Use the in-app browser to perform the logout
        var options = {
            hidden: 'yes',
            location: 'no',
            toolbar: 'no',
            fullscreen: 'yes'
        };
        $cordovaInAppBrowser.open($scope.loginData.server + '/Logout', '_blank', options);

        // Now monitor when it is done
        if (window.cordova) {
            var inAppBrowserClosed = false;
            var closeInAppBrowser = function () {
                if (!inAppBrowserClosed) {
                    inAppBrowserClosed = true;
                    $cordovaInAppBrowser.close();
                    if (offLoadStop) {
                        offLoadStop();
                    }
                    completeLogout();
                }
            }

            // Monitor when login transition is complete
            var offLoadStop = $rootScope.$on('$cordovaInAppBrowser:loadstop', function (e, event) {
                if (event.url.indexOf($scope.loginData.server) != 0) {
                    closeInAppBrowser();
                }
            });

            // Sometimes in-app browser may not definitely load, scehdule a time out of 5 sec
            $timeout(closeInAppBrowser, 5000);
        }
        else {
            cacheManager.setData("debug-csrf", undefined);
            $timeout(completeLogout, 500);
        }
    };

    if (!window.cordova) {
        // Debug specific code for mocking network status
        $scope.isOnline = true;
        $scope.toggleNetworkStatus = function () {
            if (!window.cordova) {
                // We are not running on a device.
                networkStatus.setMockNetworkStatus(!networkStatus.isOnline());
                $scope.isOnline = networkStatus.isOnline();
            }
            else {
                console.error("Attempted to toggle mock network status on a device. This should only be exposed for browser based testing.");
            }
        }
    }
});
