import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import { ngMock } from "./../../../src/ngMock";
import "./shell.js";
import { asyncDone } from "../../../src/test-utils/asyncDone.spec";

describe("Shell tests", function () {
    // TODO: duplicate of enum in shell.js, should be replaced by ref to origin enum when converting to a .ts file
    const securityCheckResult = {
        notSecure: 0,
        secure: 1,
        indeterminate: 2,
        error: 3
    };

    let scope;
    let securityCheckSuccess;
    let httpSuccess;

    let setPromiseFlags = function (isSecurityCheckSuccess, isHttpSuccess) {
        securityCheckSuccess = isSecurityCheckSuccess;
        httpSuccess = isHttpSuccess;
    };

    function createMockAppModule() {
        let getMockPromise = function (value, isSuccess) {
            if (isSuccess) {
                return Promise.resolve(value);
            } else {
                return Promise.reject(value);
            }
        };
        ngMock.module("mobile-app", function ($provide) {
            $provide.provider("deviceSecurityCheck", function () {
                this.$get = function () {
                    return {
                        checkSecurity: function () {
                            return getMockPromise(securityCheckResult.notSecure, securityCheckSuccess);
                        }
                    };
                };
            },
                $provide.provider("$http", function () {
                    this.$get = function () {
                        let httpMock: any = function () {
                            let httpServ = {
                                headers: function () {
                                    return true;
                                }
                            };
                            return getMockPromise(httpServ, httpSuccess);
                        };
                        httpMock.get = function () {
                            return Promise.resolve({});
                        };

                        return httpMock;
                    };
                })
            );
        });
        ngMock.inject(($rootScope, $controller, deviceSecurityCheck, $http) => {
            scope = $rootScope.$new();

            // stub for starting speaking logging in status
            scope.loggingInSpeaking = function () {
                return;
            };

            scope.loginData = {
                server: "https://test.server.com"
            };

            $controller("ShellCtrl", {
                $scope: scope,
                deviceSecurityCheck: deviceSecurityCheck,
                $http: $http,
            });
        });
    }
    describe("ShellCtrl", function () {
        beforeEach(createMockAppModule);

        it("should speak status on logging in process", () => {
            setPromiseFlags(true, true);
            let loggingInSpeakingCalled = false;
            scope.loggingInSpeaking = function () {
                loggingInSpeakingCalled = true;
            };
            scope.doLogin();
            expect(loggingInSpeakingCalled).to.be.true;
        });

        it("should stop speaking on success device security check", () => {
            setPromiseFlags(true, true);
            const { checkDone, markDone } = asyncDone();
            let originClearInterval = window.clearInterval;
            let clearIntervalCalled = false;
            let timerId = -1;
            scope.loggingInSpeaking = function () {
                return timerId;
            };

            window.clearInterval = function (timer) {
                if (timer === timerId) {
                    clearIntervalCalled = true;
                    markDone();
                }
            };

            return scope.doLogin().then(function () {
                checkDone();
                window.clearInterval = originClearInterval;
            });
        });

        it("should stop speaking on failed device security check", () => {
            setPromiseFlags(false, true);

            const { checkDone, markDone } = asyncDone();
            let originClearInterval = window.clearInterval;
            let clearIntervalCalled = false;
            let timerId = -1;
            scope.loggingInSpeaking = function () {
                return timerId;
            };

            window.clearInterval = function (timer) {
                if (timer === timerId) {
                    clearIntervalCalled = true;
                    markDone();
                }
            };

            return scope.doLogin().then(function () {
                checkDone();
                window.clearInterval = originClearInterval;
            });
        });

        it("should stop speaking on failed verifying of available metadata", () => {
            setPromiseFlags(true, false);

            const { checkDone, markDone } = asyncDone();
            let originClearInterval = window.clearInterval;
            let clearIntervalCalled = false;
            let timerId = -1;
            scope.loggingInSpeaking = function () {
                return timerId;
            };

            window.clearInterval = function (timer) {
                if (timer === timerId) {
                    clearIntervalCalled = true;
                    markDone();
                }
            };

            return scope.doLogin().then(function () {
                checkDone();
                window.clearInterval = originClearInterval;
            });
        });
    });
});
