import * as chai from "chai";
let expect = chai.expect;
import * as angular from "angular";
import "./task.ts";
import { ngMock } from "./../../../src/ngMock";
import * as localization from "../../../src/localization/localization";

describe("Component: Task", function () {
    let $accessibilityProvider = {
        speak: function (value) {
            return;
        },
        isScreenReaderOn: function () {
            return true;
        }
    };
    beforeEach(ngMock.module("mobile-app", function ($provide, $injector) {
        $provide.provider("$accessibility", function () {
            this.$get = function () {
                return $accessibilityProvider;
            };
        });
        $provide.provider("$page", function () {
            this.$get = function () {
                return {
                    createForm: function () {
                        return;
                    }
                };
            };
        });
    }));

    describe("with $TaskCtrl", function () {
        let controller;
        let scope;
        let accessibilitySpokenStatus;
        let onSyncStatusChangeHandler;

        beforeEach(ngMock.inject(function ($rootScope, $controller, $stateParams, $page, $ionicHistory, $navigate, $timeout, $accessibility, $label) {
            scope = $rootScope.$new();
            scope.page = {
                onSyncStatusChange: {
                    subscribe: function (callback) {
                        onSyncStatusChangeHandler = callback;
                    }
                },
                metadata: {
                    QuickSubmit: false,
                },
                syncProcessing: false,
                syncPending: false,
                syncUnitEditable: false,
                showGenericSyncError: function() {
                    return false;
                }
            };

            $accessibilityProvider.speak = function(value){
                accessibilitySpokenStatus = value;
            };

            controller = $controller("TaskCtrl",
                {
                    $scope: scope,
                    $stateParams: $stateParams,
                    $page: $page,
                    $ionicHistory: $ionicHistory,
                    $navigate:  $navigate,
                    $timeout: $timeout,
                    $accessibility: $accessibility,
                    $label: $label,
                }
            );
        }));

        it("should syncing message status be spoken", function () {
            scope.page.metadata.QuickSubmit = false;
            scope.page.syncProcessing = true;

            onSyncStatusChangeHandler();

            expect(accessibilitySpokenStatus).to.be.equal(localization.resolveLabel("Task_syncing_message"));
        });

        it("should syncing error status be spoken", function () {
            scope.page.syncProcessing = false;
            scope.page.showGenericSyncError = function() {
                return true;
            };

            onSyncStatusChangeHandler();

            expect(accessibilitySpokenStatus).to.be.equal(localization.resolveLabel("Task_syncing_error"));
        });

        it("should syncing pending status be spoken", function () {
            scope.page.metadata.QuickSubmit = false;
            scope.page.syncPending = true;
            scope.page.syncProcessing = false;
            scope.page.showGenericSyncError = function() {
                return false;
            };

            onSyncStatusChangeHandler();

            expect(accessibilitySpokenStatus).to.be.equal(localization.resolveLabel("Task_syncing_pending"));
        });

        it("should syncing completed status be spoken", function () {
            scope.page.metadata.QuickSubmit = false;
            scope.page.syncPending = false;
            scope.page.syncProcessing = false;
            scope.page.syncUnitEditable = false;
            scope.page.showGenericSyncError = function() {
                return false;
            };

            onSyncStatusChangeHandler();

            expect(accessibilitySpokenStatus).to.be.equal(localization.resolveLabel("Task_syncing_completed"));
        });
    });
});