import * as angular from "angular";
import { IFormScope, IFormServices } from "../../js/FormServices";
import { ICompleteEventArgs } from "../../../src/view-model/IPage";
import { IPageOptions } from "../../../src/view-model/page";
import { AccessibilityConstants } from "../accessibilityConstants";
interface ITaskScope extends IFormScope {
    isEditable: () => boolean;
    canGoBack: () => boolean;
    goBack: () => void;
    submit: () => void;
    taskLoaded: boolean;
}

angular.module("mobile-app.controllers").controller("TaskCtrl",
    function ($scope: ITaskScope, $stateParams, $page: IFormServices, $ionicHistory, $navigate, $timeout, $accessibility, $label) {
        let stateParams: IPageOptions = $scope.stateParams || $stateParams;
        stateParams.isEditable = true;
        let actionView;
        $page.createForm(stateParams, $scope);
        let digesting: boolean = false;
        let statusSpeakingTimeInterval;
        const taskStatusSpeaking = function () {
            if (!$scope.page.metadata.QuickSubmit && $scope.page.syncProcessing) {
                $accessibility.speak($label("Task_syncing_message"));
                statusSpeakingTimeInterval = setTimeout(function () {
                    $accessibility.speak($label("Task_syncing_message"));
                }, AccessibilityConstants.talkbackRepeatInterval);
            } else if ($scope.page.showGenericSyncError()) {
                clearInterval(statusSpeakingTimeInterval);
                $accessibility.speak($label("Task_syncing_error"));
            } else if (!$scope.page.metadata.QuickSubmit && $scope.page.syncPending && !$scope.page.syncProcessing) {
                clearInterval(statusSpeakingTimeInterval);
                $accessibility.speak($label("Task_syncing_pending"));
                statusSpeakingTimeInterval = setTimeout(function () {
                    $accessibility.speak($label("Task_syncing_pending"));
                }, AccessibilityConstants.talkbackRepeatInterval);
            } else if (!$scope.page.syncPending && !$scope.page.syncUnitEditable) {
                clearInterval(statusSpeakingTimeInterval);
                $accessibility.speak($label("Task_syncing_completed"));
            }
        };
        const onSyncStatusChangeHandler = function () {
            if (!digesting) {
                digesting = true;
                $timeout().then(function () {
                    digesting = false;
                });
            }
            if ($accessibility && $accessibility.isScreenReaderOn()) {
                taskStatusSpeaking();
            }
        };

        $scope.page.onSyncStatusChange.subscribe(onSyncStatusChangeHandler);

        $scope.isEditable = function () {
            return $scope.page.syncUnitEditable;
        };

        $scope.canGoBack = function () {
            return $ionicHistory.backView() !== undefined;
        };

        $scope.goBack = function () {
            $ionicHistory.goBack();
        };

        $scope.submit = function () {
            if ($ionicHistory.currentView) {
                actionView = $ionicHistory.currentView();
            }
            return $scope.page.submit().then((args: ICompleteEventArgs) => {
                // We don't want to stay on the task page if we didn't encounter an error
                if (actionView && actionView === $ionicHistory.currentView() && !args.error) {
                    // this promise will be resolved with either:
                    // - a navigation object after calling the OnComplete handler (if present)
                    // - or null if the handler is not present
                    // pageId should be populated from Application side to do the navigation
                    if (args && args.navigation && args.navigation.to) {
                        if (args.navigation.to.toLowerCase() === "home" || args.navigation.to.toLowerCase() === "exit"
                            || (args.navigation.params && args.navigation.params.pageId)) {
                            // Makes previous view the current view so that user navigates to list page and not
                            args.navigation.replace = true;
                            $navigate(args.navigation);
                        }
                    } else {
                        $ionicHistory.goBack();
                    }
                }
            });
        };
        $scope.taskLoaded = true;
    }
);
