﻿(function () {
    'use strict';

    angular.module('mobile-app').component('timer', {
        bindings: {
            metadata: '=',
            isEditable: '<',
            setValue: '&',
            getValue: '&',
            stateParams: '=',
        },
        controller: function ($scope, $filter, $stateParams, cacheManager, $interval) {

            $scope.Timer = function (controlMetadata) {

                var controlId = controlMetadata.Id;
                var timerControl = this;
                var cacheKeyLastStartTimeInMs = "pbl-" + $stateParams.appId + "-" + "timerLastStartTimeInMs" + "-" + controlId + "-" + $stateParams.pageContext;
                var cacheKeyLastPausedDurationInMs = "pbl-" + $stateParams.appId + "-" + "timerLastPausedDurationInMs" + "-" + controlId + "-" + $stateParams.pageContext;
                var cacheKeyRunning = "pbl-" + $stateParams.appId + "-" + "timerRunning" + "-" + controlId + "-" + $stateParams.pageContext;
                var cacheKeyStopped = "pbl-" + $stateParams.appId + "-" + "timerStopped" + "-" + controlId + "-" + $stateParams.pageContext;
                var cacheKeyInitializationStartTimeInMs = "pbl-" + $stateParams.appId + "-" + "timerInitializationStartTimeInMs" + "-" + controlId + "-" + $stateParams.pageContext;
                var ctrlData = timerControl.value = $scope.$parent.$ctrl.getValue();

                var initialization = {
                    HideSecondsWhenStopped: false,
                    durationInHours: false,
                    minimumDurationMinutes: 1,
                    hideTimerButton: false
                }

                if (controlMetadata.OnInitialize) {
                    controlMetadata.OnInitialize(initialization || {})
                    ctrlData = initialization.durationInHours ? Math.round(ctrlData * 60) : ctrlData;
                }

                timerControl.Options = { Hours: [], Minutes: [] };
                timerControl.HideSecondsWhenStopped = initialization.HideSecondsWhenStopped;

                timerControl.ID = null;
                timerControl._lastStartTimeInsMs = cacheManager.getData(cacheKeyLastStartTimeInMs) ? cacheManager.getData(cacheKeyLastStartTimeInMs) : 0;

                timerControl._lastPausedDurationInMs = cacheManager.getData(cacheKeyLastPausedDurationInMs) ? cacheManager.getData(cacheKeyLastPausedDurationInMs) : 0;
                // if previous initialization start time is zero than timer is starting for the first time. Initialize timer to start time passed by caller.
                // the only other time this if is expected to be true is if the start time passed from the caller has changed
                var previousInitializationStartTimeInMs = cacheManager.getData(cacheKeyInitializationStartTimeInMs) ? cacheManager.getData(cacheKeyInitializationStartTimeInMs) : 0;
                if (ctrlData && previousInitializationStartTimeInMs != ctrlData * 60 * 1000) { //timerControl._lastPausedDurationInMs == 0 ||
                    timerControl._lastPausedDurationInMs = ctrlData * 60 * 1000;
                    cacheManagerSetData(cacheKeyLastPausedDurationInMs, timerControl._lastPausedDurationInMs);
                    cacheManagerSetData(cacheKeyInitializationStartTimeInMs, timerControl._lastPausedDurationInMs);
                    // reset the other caches to initial values
                    cacheManagerSetData(cacheKeyLastStartTimeInMs, 0);
                    cacheManagerSetData(cacheKeyRunning, false);
                    cacheManagerSetData(cacheKeyStopped, false);
                }

                timerControl.IsRunning = cacheManager.getData(cacheKeyRunning) ? cacheManager.getData(cacheKeyRunning) : false;
                timerControl._runningTimeInMs = timerControl.IsRunning ? Date.now() - timerControl._lastStartTimeInsMs + timerControl._lastPausedDurationInMs : timerControl._lastPausedDurationInMs;
                // do not allow timer to run over 23 hours 59 minutes 59 seconds
                if (timerControl._runningTimeInMs >= (23 * 60 * 60 * 1000) + (59 * 60 * 1000) + (59 * 1000)) {
                    timerControl._runningTimeInMs = (23 * 60 * 60 * 1000) + (59 * 60 * 1000) + (59 * 1000);
                }

                timerControl.IsStopped = cacheManager.getData(cacheKeyStopped) ? cacheManager.getData(cacheKeyStopped) : false;
                timerControl.Duration = ctrlData ? ctrlData : 0;
                timerControl.days = 0;
                timerControl.hours = 0;
                timerControl.Hours = "00";
                timerControl.Minutes = "00";
                timerControl.Seconds = "00";
                timerControl.minutes = 0;
                timerControl.seconds = 0;

                timerControl.Start = function () {
                    if ($stateParams && $stateParams.pageContext != null) {
                        timerControl.ID = $interval(timerControl._Tick, 1000);
                        timerControl.IsRunning = true;
                        cacheManagerSetData(cacheKeyRunning, timerControl.IsRunning);
                        timerControl._lastStartTimeInsMs = Date.now();
                        cacheManagerSetData(cacheKeyLastStartTimeInMs, timerControl._lastStartTimeInsMs);
                        if (controlMetadata.OnTimerStateChanged) {
                            controlMetadata.OnTimerStateChanged(timerControl.IsRunning);
                        }
                    }
                    else {
                        // show warning box that record must be saved before timer can be started.
                        document.getElementById("timerWarningMessage").innerHTML = "The record must be saved before you can start the timer.";
                        document.getElementById("timerWarningMessage").style.visibility = "visible";
                    }
                };

                timerControl.ClearWarning = function () {
                    // hide warning box
                    document.getElementById("timerWarningMessage").innerHTML = "";
                    document.getElementById("timerWarningMessage").style.visibility = "hidden";
                }

                timerControl.Pause = function () {
                    if (timerControl.ID) {
                        $interval.cancel(timerControl.ID);
                        timerControl.IsRunning = false;
                        if (controlMetadata.OnTimerStateChanged) {
                            controlMetadata.OnTimerStateChanged(timerControl.IsRunning);
                        }
                        cacheManagerSetData(cacheKeyRunning, timerControl.IsRunning);
                        timerControl.Hours = formatValue(timerControl.hours);
                        timerControl.Minutes = formatValue(timerControl.minutes);
                        timerControl._lastPausedDurationInMs = timerControl._runningTimeInMs;
                        cacheManagerSetData(cacheKeyLastPausedDurationInMs, timerControl._lastPausedDurationInMs);
                    }
                };
                timerControl.Stop = function () {
                    if (timerControl.ID) {
                        timerControl.Pause();
                        //timerControl.Duration = parseInt(timerControl.Duration) + Math.floor(timerControl._runningTimeInMs / (1000 * 60)); // convert milliseconds to minutes
                        timerControl.Duration = Math.floor(timerControl._runningTimeInMs / (1000 * 60)); // convert milliseconds to minutes

                        $scope.$parent.$ctrl.value = timerControl.Duration;
                        $scope.$parent.$ctrl.setValue({ newvalue: $scope.$parent.$ctrl.value });

                        timerControl.IsStopped = true;
                        cacheManagerSetData(cacheKeyStopped, timerControl.IsStopped);
                    }
                };
                timerControl._UpdateDisplayedTime = function () {
                    var timerDate = new Date();
                    timerDate.setTime(timerControl._runningTimeInMs);
                    var days = Math.floor(timerControl._runningTimeInMs / (24 * 60 * 60 * 1000));

                    if (timerControl.days != days) {
                        timerControl.days = days;
                    }
                    if (timerControl.hours != timerDate.getUTCHours()) {
                        timerControl.hours = timerDate.getUTCHours();
                        timerControl.Hours = formatValue(timerControl.hours);
                    }
                    if (timerControl.minutes != timerDate.getUTCMinutes()) {
                        timerControl.minutes = timerDate.getUTCMinutes();
                        timerControl.Minutes = formatValue(timerControl.minutes);
                    }
                    if (timerControl.seconds != timerDate.getUTCSeconds()) {
                        timerControl.seconds = timerDate.getUTCSeconds();
                        timerControl.Seconds = formatValue(timerControl.seconds);
                    }
                };
                timerControl._Tick = function () {
                    if (timerControl.ID) {
                        if (timerControl._runningTimeInMs < 0) { $interval.cancel(timerControl.ID); }

                        // do not allow timer to run over 23 hours 59 minutes 59 seconds
                        if (timerControl._runningTimeInMs >= (23 * 60 * 60 * 1000) + (59 * 60 * 1000) + (59 * 1000)) {
                            $interval.cancel(timerControl.ID);
                        }
                        else {
                            timerControl._runningTimeInMs += 1000;
                        }
                        timerControl._UpdateDisplayedTime();
                    }
                };
                timerControl.HandleValueChange = function () {
                    timerControl.seconds = 0;
                    timerControl.Seconds = "00";
                    timerControl._runningTimeInMs = (parseInt(timerControl.Hours) * 60 * 60 * 1000) + (parseInt(timerControl.Minutes) * 60 * 1000) + timerControl.seconds * 1000;
                    cacheManagerSetData(cacheKeyLastPausedDurationInMs, timerControl._runningTimeInMs);

                    $scope.$parent.$ctrl.value = Math.floor(timerControl._runningTimeInMs / (1000 * 60)); // convert milliseconds to minutes
                    $scope.$parent.$ctrl.setValue({ newvalue: $scope.$parent.$ctrl.value });

                    timerControl._UpdateDisplayedTime();
                };

                function formatValue(time) {
                    return time < 10 ? '0' + time : time;
                }

                function cacheManagerSetData(cacheKey, data) {
                    if ($stateParams.pageContext != null) {
                        cacheManager.setData(cacheKey, data);
                    }
                };

                // initialize the timer display values
                // Populate options for hour
                for (var i = 0; i < 24; i++) {
                    timerControl.Options.Hours.push(formatValue(i));
                }
                // Populate options for hour
                for (var i = 0; i < 60; i = i + parseInt(initialization.minimumDurationMinutes)) {
                    timerControl.Options.Minutes.push(formatValue(i));
                }

                // Render timer based on defaults or cache
                timerControl._UpdateDisplayedTime();

                // If cache says timer was previously running, then start the animation
                if (timerControl.IsRunning) {
                    timerControl.ID = $interval(timerControl._Tick, 1000);
                    if (controlMetadata.OnTimerStateChanged) {
                        controlMetadata.OnTimerStateChanged(timerControl.IsRunning);
                    }
                }

                return timerControl;
            };
        },
        templateUrl: 'components/timer/timer.html',
    });
})();
