import * as angular from "angular";
import { defer } from "../../src/defer";
interface IPageModalOptions {
    scope: angular.IScope;
    hardwareBackButtonClose: boolean;
    useDefaultStatusBar: boolean; //Donot change the status bar
}

angular.module("mobile-app.services").factory("$pageModal", function (
    $compile, $document, $ionicPlatform, $cordovaStatusbar
) {
    const modalTemplate: string = `<div class="page-modal-backdrop"><div class="page-modal page-modal-hidden">
    <div ng-include="\'$templateURL\'" onload="onLoad()"></div></div></div>`;
    return {
        fromTemplateUrl(templateUrl: string, options: IPageModalOptions) {
            let template;
            let modalContainer;

            let modal = {
                show: function (): Promise<void> {
                    return new Promise<void>(function (resolve, reject) {
                        // change status bar content color to contrast with modal background
                        if (options.useDefaultStatusBar === false && (window as any).StatusBar) {
                            // styles: Default : 0, LightContent: 1, BlackTranslucent: 2, BlackOpaque: 3

                            $cordovaStatusbar.style(0);
                        }

                        const modalScope = options.scope.$new();

                        (modalScope as any).onLoad = function () {

                            modalContainer = template.children()[0];
                            // carry out tasks outside the event loop
                            setTimeout(() => {

                                if (template) {

                                    modalContainer.classList.remove("page-modal-hidden");
                                    resolve();
                                }
                            });
                        };

                        template = angular.element(modalTemplate.replace("$templateURL", templateUrl));
                        angular.element(document.body).append(template);
                        $compile(template)(modalScope);

                        if (options.hardwareBackButtonClose) {
                            $ionicPlatform.onHardwareBackButton(function () {
                                modal.hide().then(function () {
                                    modal.remove();
                                });
                                options.scope.$broadcast("modal.hidden");
                            });
                        }
                    });
                },
                hide: function (): Promise<void> {
                    // reset status bar content color
                    if (options.useDefaultStatusBar === false && (window as any).StatusBar) {
                        // styles: Default : 0, LightContent: 1, BlackTranslucent: 2, BlackOpaque: 3
                        $cordovaStatusbar.style(1);
                    }

                    modalContainer.classList.add("page-modal-hidden");

                    // set a delayed promise resolve to accommodate animation
                    return new Promise<void>((resolve) => {
                        setTimeout(() => {
                            resolve();
                        }, 350);
                    });
                },
                remove: function () {
                    template.remove();
                }
            };
            let modalResult = defer<any>();
            modalResult.resolve(modal);
            return modalResult.promise;
        }
    };
});