import * as angular from "angular";
import { PageType } from "../../src/view-model/pageType";
import { MocksHelper } from "../../src/mobile-app-tests/reservationMgmt/mocks/reservation-mocks-helper.mocks";

export function registerStates() {
    angular.module("mobile-app").config(function ($ionicConfigProvider, $stateProvider, $urlRouterProvider) {
        // Keep the back text as always being empty string on all platforms
        $ionicConfigProvider.backButton.previousTitleText(false);
        $ionicConfigProvider.backButton.text("");

        // Check whether mocks should be used
        if ("production" !== process.env.NODE_ENV) {
            let mocksHelper = new MocksHelper();
            let mocksRegex = new RegExp("[?&]useMocks(=([^&#]*)|&|#|$)");
            let mocksResult = mocksRegex.exec(document.URL);

            if (mocksResult && mocksResult[2] === "true") {
                mocksHelper.turnOnMocks();
            }
        }

        $stateProvider
            .state("app", {
                url: "/app",
                abstract: true,
                templateUrl: "components/shell/shell.html",
                controller: "ShellCtrl"
            })
            .state("app.notifications", {
                url: "/notifications/:entityContext/:status",
                params: {
                    entityContext: {
                        value: null,
                        squash: true
                    },
                    status: {
                        value: null,
                        squash: true,
                    }

                },
                views: {
                    "menuContent": {
                        templateUrl: "components/notifications-hub/notifications-hub.html",
                        controller: "NotificationsHubCtrl"
                    }
                }
            })

            .state("app.appList", {
                url: "/appList",
                params: {
                    allowedStaleness: {
                        value: 20000,
                    }
                },
                views: {
                    "menuContent": {
                        templateUrl: "components/app-list/app-list.html",
                        controller: "AppListCtrl"
                    }
                }
            })

            .state("app.appLanding", {
                url: "/appLanding/:appId",
                views: {
                    "menuContent": {
                        templateUrl: "components/detail-page/detail-page.html",
                        controller: "AppLandingCtrl"
                    }
                }
            })

            .state(PageType.detailPage, {
                url: "/detailPage/:appId/:pageId/:pageContext",
                params: {
                    pageContext: {
                        value: null,
                        squash: true
                    },
                    filter: {
                        value: null,
                        squash: true
                    }
                },
                views: {
                    "menuContent": {
                        templateUrl: "components/detail-page/detail-page.html",
                        controller: "DetailPageCtrl"
                    }
                }
            })

            .state(PageType.task, {
                url: "/task/:appId/:pageId/:pageContext",
                params: {
                    pageContext: {
                        value: null,
                        squash: true
                    },
                    hiddenState: {
                        value: null,
                        squash: true
                    },
                    queryPath: null

                },
                views: {
                    "menuContent": {
                        templateUrl: "components/task/task.html",
                        controller: "TaskCtrl"
                    }
                },
                disableHardwareBackButton: true
            })

            .state("app.taskInstance", {
                url: "/taskInstance/:appId/:pageId/:jobInstanceId/:pageContext",
                params: {
                    pageContext: {
                        value: null,
                        squash: true
                    },
                },
                views: {
                    "menuContent": {
                        templateUrl: "components/task/task.html",
                        controller: "TaskCtrl"
                    }
                },
                disableHardwareBackButton: true
            });

        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise("/app/appList");
    });
}