require("core-js");
require("../../src/custom-event");
require("../../scss/dynamics-ax.app.scss");

// Load angular and angular dependencies
if ("production" !== process.env.NODE_ENV) {
    require("ionic/js/ionic.bundle.js");
    require("ngCordova/dist/ng-cordova.js");
} else {
    require("ionic/js/ionic.bundle.min.js");
    require("ngCordova/dist/ng-cordova.min.js");
}

import * as angular from "angular";
import { registerApp, initializeApp } from "./app-module-setup";
import { registerStates } from "./app-states-config";
import * as telemetry from "../../src/services/telemetry";
let fileTransferProvider = require("@ax/services/file-transfer-provider");
import { cacheManager } from "@ms-cds/data-model";
import { registerAllControls } from "../../src/view-model/Control/registration";
import { registerAllRendererComponents } from "../components/control/rendererRegistration";
import { IConnector } from "../../src/connectors/IConnector";

// register the app module and controllers
registerApp();
// register and configure states
registerStates();

// Wait until the document has loaded
angular.element(document).ready(function () {
    if ((window as any).cordova) {
        // We are running inside cordova, rely on 'deviceready' to fire
        document.addEventListener("deviceready", handleDeviceReady, false);
    } else {
        // Running in browser, so trigger device ready manually
        handleDeviceReady();
    }
});

/**
 * Handles the device ready for the app
 */
function handleDeviceReady() {
    // Initialize the file based caching
    telemetry.trackEvent(telemetry.events.initializeFileStoreStarted.Name);
    cacheManager.initializeFileStore()
        .then(() => {
            // Initialize the app, and perform security check on start
            telemetry.trackEvent(telemetry.events.initializeFileStoreEnded.Name);
            initializeApp(true);
            telemetry.trackEvent(telemetry.events.initializeAppStepCompleted.Name);
        })
        .then(() => {
            //Register controls
            registerAllControls();
            //Register renderer controls
            registerAllRendererComponents();

            // Load page component
            require("../components/detail-page/detail-page.ts");
            // Load task component
            require("../components/task/task.ts");
            // Load change company component
            require("../components/company-selector/company-selector.js");

            // Now start the angular application
            telemetry.trackEvent(telemetry.events.angularBootstrapStarted.Name);
            angular.bootstrap(document, ["mobile-app"]);
            telemetry.trackEvent(telemetry.events.angularBootstrapEnded.Name);

            // Wait a second and then hide the splashscreen as the app is ready
            if ((navigator as any).splashscreen) {
                window.setTimeout(function () {
                    (navigator as any).splashscreen.hide();
                }, 1000);
            }
        });
}

function handleWindowResize(windowSizeReduced: boolean) {
    if (windowSizeReduced) {
        if (!angular.element(document.body).hasClass("window-size-reduced")) {
            angular.element(document.body).addClass("window-size-reduced");
        }
    } else if (angular.element(document.body).hasClass("window-size-reduced")) {
        angular.element(document.body).removeClass("window-size-reduced");
    }
}

angular.module("mobile-app")
    .run(function ($ionicPlatform, $cordovaStatusbar) {
        $ionicPlatform.ready(function () {
            if ((window as any).cordova && (window as any).cordova.platformId === "android") {
                let innerHeight: number = window.innerHeight;
                window.onresize = function (event) {
                    let currentHeight = window.innerHeight;
                    if (currentHeight < innerHeight) {
                        handleWindowResize(true);
                    } else {
                        handleWindowResize(false);
                    }
                };
            }
            // set IOS Status Bar
            if ((window as any).StatusBar) {
                // styles: Default : 0, LightContent: 1, BlackTranslucent: 2, BlackOpaque: 3
                $cordovaStatusbar.style(1);
            }
            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            if ((window as any).cordova && (window as any).cordova.platformId === "ios" && (window as any).cordova.plugins.Keyboard) {
                (window as any).cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                (window as any).cordova.plugins.Keyboard.disableScroll(true);

                window.addEventListener("native.keyboardshow", function (event) {
                    handleWindowResize(true);
                    const focusedElement = document.activeElement;
                    if (focusedElement) {
                        let element = angular.element(focusedElement);
                        // find enclosing 'control' parent
                        const className = "control";
                        let focusableElementFound = false;
                        let level = 0;
                        // Look for upto 10 levels up
                        while (element && element[0] && level < 10) {
                            if (element.hasClass(className)) {
                                focusableElementFound = true;
                                break;
                            }
                            element = element.parent && element.parent();
                            level++;
                        }

                        if (focusableElementFound && element && element[0]) {
                            // first reduce the view height to accomodate the keyboard
                            let keyboardHeight = (event as any).keyboardHeight + "px";
                            angular
                                .element(document.body)
                                .css({ position: "fixed", bottom: keyboardHeight });
                            // then scroll the enclosing input parent into view
                            let elementDOM = element[0];
                            elementDOM.scrollIntoView(false);
                        }
                    }
                });

                window.addEventListener("native.keyboardhide", function (event) {
                    handleWindowResize(false);
                    // we dont have to change the bottom property as it doesn't affect statically positioned elements
                    angular.element(document.body).css({
                        position: "static"
                    });
                });
            }
        });
    })

    .run(function ($cordovaFileTransfer) {
        fileTransferProvider.set($cordovaFileTransfer);
    });

// Load our angular components into the angular modules. This needs to be done after the angular modules have been declared.
require("./components");